<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; either 
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; 
if not, write to the 
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2013
sirfragalot@users.sourceforge.net
**********************************************************************************/
/*****
*	ADMINCATEGORIES class.
*
*	Administration of categories and subcategories
*****/
class ADMINCATEGORIES
{
private $db;
private $vars;
private $errors;
private $messages;
private $success;
private $session;
private $config;
private $form;
private $keyword;
private $html;
private $gatekeep;
private $badInput;
private $categories;
private $table;
private $ajax;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->success = FACTORY_SUCCESS::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->config = FACTORY_CONFIG::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->category = FACTORY_CATEGORY::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->gatekeep = FACTORY_GATEKEEP::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->ajax = FACTORY_AJAX::getInstance();
		$this->gatekeep->requireSuper = TRUE;
		$this->gatekeep->init();
		$this->session->clearArray('edit');
		$this->categories = $this->category->grabAll();
		if(empty($this->categories))
		{
			GLOBALS::buildOutputString($pString . $this->messages->text("misc", "noCategories"));
			return;
		}
// Administering 'General' category (key == 1) is not allowed.
		unset($this->categories[1]);
	}
// Add a category - display options.
	public function catInit($message = FALSE)
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "editCategory"));
		$pString = $message ? $this->html->p($message, "error", "center") : FALSE;
		$pString .= $this->table->tableStart('generalTable', 1, 0, 0, "left");
		$pString .= $this->table->trStart();
// Add
		$td = $this->form->formHeader("admin_ADMINCATEGORIES_CORE");
		$td .= $this->form->hidden("method", "addCat");
		$td .= $this->form->textInput($this->messages->text("category", "addCategory"), "categoryAdd", FALSE, 30, 255);
		$td .= $this->html->p($this->form->formSubmit('Add'));
		$td .= $this->form->formEnd();
		$pString .= $this->table->td($td);
		if(!empty($this->categories))
		{
// Edit
// If preferences reduce long categories, we want to transfer the original rather than the condensed version. 
// Store the base64-encoded value for retrieval in the javascript.
			foreach($this->categories as $key => $value)
			{
				$key = $key . '_' . base64_encode($value);
				$categories[$key] = $value;
			}
			$td = $this->form->formHeader("admin_ADMINCATEGORIES_CORE");
			$td .= $this->form->hidden("method", "editCat");
			$td .= $this->form->selectFBoxValue($this->messages->text("category", "editCategory"), 
				'categoryId', $categories, 10);
			$td .= $this->html->p($this->transferArrow('transferCategory'));
			$td .= $this->html->p($this->form->textInput(FALSE, "categoryEdit", FALSE, 30, 255));
			$td .= $this->form->hidden('categoryEditId', FALSE);
			$td .= $this->html->p($this->form->formSubmit('Edit'));
			$td .= $this->form->formEnd();
			$pString .= $this->table->td($td);
// Delete
			$td = $this->form->formHeader("admin_ADMINCATEGORIES_CORE");
			$td .= $this->form->hidden("method", "deleteCatConfirm");
			$td .= $this->form->selectFBoxValueMultiple($this->messages->text("category", "deleteCategory"), 
				'categoryIds', $this->categories, 10) . $this->html->br() . $this->html->span($this->messages->text("hint", "multiples"), 'hint');
			$td .= $this->html->p($this->messages->text("category", "deleteWarning"));
			$td .= $this->html->p($this->form->formSubmit('Delete'));
			$td .= $this->form->formEnd();
			$pString .= $this->table->td($td);
		}
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$this->ajax->loadJavascript(array('core/modules/admin/categoryEdit.js'));
		GLOBALS::buildOutputString($pString);
	}
// Add a subcategory - display options.
	public function subInit($message = FALSE)
	{
		if(empty($this->categories))
		{
			GLOBALS::buildOutputString($pString . $this->messages->text("misc", "noCategories"));
			return;
		}
		GLOBALS::setOutputHeading($this->messages->text("heading", "editSubcategory"));
		$pString = $message ? $this->html->p($message, "error", "center") : FALSE;
		$pString .= $this->table->tableStart('generalTable', 1, 0, 0, "left");
		$pString .= $this->table->trStart();
// Add
		$td = $this->form->formHeader("admin_ADMINCATEGORIES_CORE");
		$td .= $this->form->hidden("method", "addSub");
		$td .= $this->form->textInput($this->messages->text("category", "addSubcategory"), "addSubcategory", FALSE, 30, 255);
		$categories = $this->category->grabAll();
		$td .= $this->html->p($this->form->selectFBoxValue($this->messages->text('resources', 'subcategoryPart'), 
			'categoryId', $categories, 10));
		$td .= $this->html->p($this->form->formSubmit('Add'));
		$td .= $this->form->formEnd();
		$pString .= $this->table->td($td);
// Edit
		$subcategories = $this->category->grabSubAll();
		if(is_array($subcategories))
		{
			$td = $this->form->formHeader("admin_ADMINCATEGORIES_CORE");
			$td .= $this->form->hidden("method", "editSub");
			$jScript = 'index.php?action=admin_ADMINCATEGORIES_CORE&method=subcatIsPartCat';
// Amend category list depending upon which subcategory is chosen
			$jsonArray[] = array(
				'startFunction' => 'triggerFromSelect', 
				'script' => "$jScript",
				'triggerField' => 'subcategoryId',
				'targetDiv' => 'categoryIdDiv',
				);
			$this->ajax->jActionForm('onclick', $jsonArray);
// If preferences reduce long subcategories, we want to transfer the original rather than the condensed version. 
// Store the base64-encoded value for retrieval in the javascript.
			foreach($subcategories as $key => $value)
			{
				$key = $key . '_' . base64_encode($value);
				$subcats[$key] = $value;
			}
			$td1 = $this->table->td($this->form->selectFBoxValue($this->messages->text("category", "editSubcategory"), 
				'subcategoryId', $subcats, 10));
			$this->db->formatConditions(array('subcategoryId' => array_shift(array_keys($subcategories))));
			$selected = $this->db->fetchOne($this->db->select('subcategory', 'subcategoryCategoryId'));
			$td1 .= $this->table->td($this->html->div('categoryIdDiv', $this->form->selectedBoxValue($this->messages->text('resources', 'subcategoryPart'), 
				'categoryId', $categories, $selected, 10)));
			$td2 = $this->html->p($this->transferArrow('transferSubcategory'));
			$td2 .= $this->html->p($this->form->textInput(FALSE, "subcategoryEdit", FALSE, 30, 255));
			$td2 .= $this->form->hidden('subcategoryEditId', FALSE);
			$td2 .= $this->html->p($this->form->formSubmit('Edit'));
			$td .= $this->table->tableStart();
			$td .= $this->table->trStart();
			$td .= $td1;
			$td .= $this->table->trEnd();
			$td .= $this->table->trStart();
			$td .= $this->table->td($td2, FALSE, FALSE, FALSE, 2);
			$td .= $this->table->trEnd();
			$td .= $this->table->tableEnd();
			$td .= $this->form->formEnd();
			$pString .= $this->table->td($td);
// Delete
			$td = $this->form->formHeader("admin_ADMINCATEGORIES_CORE");
			$td .= $this->form->hidden("method", "deleteSubcatConfirm");
			$td .= $this->form->selectFBoxValueMultiple($this->messages->text("category", "deleteSubcategory"), 
				'subcategoryIds', $subcategories, 10) . $this->html->br() . $this->html->span($this->messages->text("hint", "multiples"), 'hint');
			$td .= $this->html->p($this->form->formSubmit('Delete'));
			$td .= $this->form->formEnd();
			$pString .= $this->table->td($td);
		}
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$this->ajax->loadJavascript(array('core/modules/admin/categoryEdit.js'));
		GLOBALS::buildOutputString($pString);
	}
// Amend category select box depening on selection in subcategory select box
	public function subcatIsPartCat()
	{
		$this->db->formatConditions(array('subcategoryId' => $this->vars['ajaxReturn']));
		$selected = $this->db->fetchOne($this->db->select('subcategory', 'subcategoryCategoryId'));
		$categories = $this->category->grabAll();
		$div = $this->form->selectedBoxValue($this->messages->text('resources', 'subcategoryPart'), 
			'categoryId', $categories, $selected, 10);
		GLOBALS::buildOutputString($this->ajax->encode_jArray(array('innerHTML' => $div)));
		FACTORY_CLOSERAW::getInstance();
	}
// Add a category
	public function addCat()
	{
		if(!$input = $this->validateInput('addCat'))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'catInit');
// database match is case insensitive.
		$this->db->formatConditions(array('categoryCategory' => $input));
		$recordset = $this->db->select('category', 'categoryId');
		$pString = $this->success->text("category", " " . $this->messages->text("misc", "added") . " ");
// If category already exists quietly return without error.
		if($id = $this->db->fetchOne($recordset))
			return $this->catInit($pString);
		$this->db->insert('category', 'categoryCategory', $input);
		$this->categories = $this->category->grabAll();
		unset($this->categories[1]);
		return $this->catInit($pString);
	}
// Add a subcategory
	public function addSub()
	{
		if(!$input = $this->validateInput('addSub'))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'subInit');
// database match is case insensitive.
		$this->db->formatConditions(array('subcategorySubcategory' => $input['addSubcategory']));
		$this->db->formatConditions(array('subcategoryCategoryId' => $input['categoryId']));
		$recordset = $this->db->select('subcategory', 'subcategoryId');
		$pString = $this->success->text("subcategory", " " . $this->messages->text("misc", "added") . " ");
// If subcategory already exists quietly return without error.
		if($id = $this->db->fetchOne($recordset))
			return $this->subInit($pString);
		$fields[] = 'subcategorySubcategory';
		$values[] = $input['addSubcategory'];
		$fields[] = 'subcategoryCategoryId';
		$values[] = $input['categoryId'];
		$this->db->insert('subcategory', $fields, $values);
		return $this->subInit($pString);
	}
// Ask for confirmation of delete categories
	public function deleteCatConfirm()
	{
		if(!$input = $this->validateInput('deleteCat'))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'catInit');
		GLOBALS::setOutputHeading($this->messages->text("heading", "editCategory"));
		$categories = "'" . implode("', '", array_keys(array_intersect(array_flip($this->categories), $input))) . "'";
		$categories = html_entity_decode($categories);
		$pString = $this->html->p($this->messages->text("category", "deleteCatConfirm", ": $categories"));
		$pString .= $this->form->formHeader("admin_ADMINCATEGORIES_CORE");
		$pString .= $this->form->hidden("method", "deleteCat");
		$pString .= $this->form->hidden('categoryIds', base64_encode(serialize($input)));
		$pString .= $this->html->br() . $this->form->formSubmit('Confirm');
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// Ask for confirmation of delete subcategories
	public function deleteSubCatConfirm()
	{
		if(!$input = $this->validateInput('deleteSub'))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'subInit');
		GLOBALS::setOutputHeading($this->messages->text("heading", "editSubcategory"));
		$this->db->formatConditionsOneField($input, 'subcategoryId');
		$recordset = $this->db->select('subcategory', 'subcategorySubCategory');
		while($row = $this->db->fetchRow($recordset))
			$list[] = $this->html->dbToHtmlTidy($row['subcategorySubCategory']);
		$subcategories = html_entity_decode(implode(', ', $list));
		$pString = $this->html->p($this->messages->text("category", "deleteSubConfirm", ": $subcategories"));
		$pString .= $this->form->formHeader("admin_ADMINCATEGORIES_CORE");
		$pString .= $this->form->hidden("method", "deleteSub");
		$pString .= $this->form->hidden('subcategoryIds', base64_encode(serialize($input)));
		$pString .= $this->html->br() . $this->form->formSubmit('Confirm');
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// Delete categories
	public function deleteCat()
	{
		if(!$input = $this->validateInput('deleteCatConfirm'))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'catInit');
// ensure that category 1 'General' is never deleted
		if($key = array_search(1, $input))
			unset($input[$key]);
		if(!$this->deleteSql($input))
			$this->badInput($this->errors->text("inputError", "invalid"), 'catInit');
		$pString = $this->success->text("category", " " . $this->messages->text("misc", "deleted") . " ");
		$this->categories = $this->category->grabAll();
		unset($this->categories[1]);
		return $this->catInit($pString);
	}
// Delete subcategories
	public function deleteSub()
	{
		if(!$input = $this->validateInput('deleteSubConfirm'))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'subInit');
		if(!$this->deleteSubSql($input))
			$this->badInput($this->errors->text("inputError", "invalid"), 'subInit');
		$pString = $this->success->text("category", " " . $this->messages->text("misc", "deleted") . " ");
		return $this->subInit($pString);
	}
// Delete category(s) from category table.  $input is array of ids in WKX_category.
// 'General' category always has id 1.
	private function deleteSql($input)
	{
// Delete resource_category rows having the category IDs
		$this->db->formatConditionsOneField($input, 'resourcecategoryCategoryId');
		$this->db->delete('resource_category');
// Find subcategories that are part of the categories and delete those rows from resource_category
		$this->db->formatConditionsOneField($input, 'subcategoryCategoryId');
		$recordset = $this->db->select('subcategory', 'subcategoryId');
		while($row = $this->db->fetchRow($recordset))
			$subcatIds[] = $row['subcategoryId'];
		if(isset($subcatIds))
		{
			$this->db->formatConditionsOneField($subcatIds, 'resourcecategorySubcategoryId');
			$this->db->delete('resource_category');
		}
// Ensure that all resources not listed in resource_category have at least the 'General' category
		$subStmt = $this->db->subQuery($this->db->selectNoExecute('resource_category', 'resourcecategoryResourceId'), FALSE, FALSE, TRUE);
		$this->db->formatConditions($this->db->formatFields('resourceId'). $this->db->inClause($subStmt, TRUE));
		$recordset = $this->db->select('resource', 'resourceId');
		while($row = $this->db->fetchRow($recordset))
			$this->db->insert('resource_category', array('resourcecategoryResourceId', 'resourcecategoryCategoryId'), array($row['resourceId'], '1'));
// Finally delete categories and any subcategories and optimize tables
		foreach($input as $value)
		{
			$this->db->formatConditions(array('categoryId' => $value));
			$this->db->delete('category');
			$this->db->formatConditions(array('subcategoryCategoryId' => $value));
			$this->db->delete('subcategory');
		}
		$this->db->optimize('category');
		$this->db->optimize('subcategory');
		$this->db->optimize('resource_category');
		return TRUE;
	}
// Delete subcategory(s) from subcategory table and from subcategories field in WKX_resource_category.  
// $input is array of ids in WKX_subcategory.
	private function deleteSubSql($input)
	{
// Delete resource_category rows having the subcategory IDs
		$this->db->formatConditionsOneField($input, 'resourcecategorySubcategoryId');
		$this->db->delete('resource_category');
// Delete subcategories and optimize tables
		foreach($input as $value)
		{
			$this->db->formatConditions(array('subcategoryId' => $value));
			$this->db->delete('subcategory');
		}
		$this->db->optimize('resource_category');
		$this->db->optimize('subcategory');
		return TRUE;
	}
// Edit categories
	public function editCat()
	{
		if(!$input = $this->validateInput('editCat'))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'catInit');
		$this->db->formatConditions(array('categoryId' => $input['id']));
		$recordset = $this->db->select('category', 'categoryId');
		if(!$id = $this->db->fetchOne($recordset))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'catInit');
		$update['categoryCategory'] = $input['text'];
		$this->db->formatConditions(array('categoryId' => $input['id']));
		$this->db->update('category', $update);
		$pString = $this->success->text("category", " " . $this->messages->text("misc", "edited") . " ");
		$this->categories = $this->category->grabAll();
		unset($this->categories[1]);
		return $this->catInit($pString);
	}
// Edit a subcategory
	public function editSub()
	{
		if(!$input = $this->validateInput('editSub'))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'subInit');
		$this->db->formatConditions(array('subcategoryId' => $input['subcategoryEditId']));
		$oldCatId = $this->db->fetchOne($this->db->select('subcategory', 'subcategoryCategoryId'));
// Need to insert new rows to resource_category if the category has changed -- get resource Id from this subCategory
		if($oldCatId != $input['categoryId'])
		{
			$this->db->formatConditions(array('resourcecategorySubcategoryId' => $input['subcategoryEditId']));
			$resultset = $this->db->select('resource_category', array('resourcecategoryCategoryId', 'resourcecategoryResourceId'), TRUE);
			while($row = $this->db->fetchRow($resultset))
			{
				$this->db->formatConditions(array('resourcecategoryCategoryId' => $input['categoryId']));
				$this->db->formatConditions(array('resourcecategoryResourceId' => $row['resourcecategoryResourceId']));
				if(!$this->db->numRows($this->db->select('resource_category', '*')))
					$this->db->insert('resource_category', array('resourcecategoryResourceId', 'resourcecategoryCategoryId'), 
						array($row['resourcecategoryResourceId'], $input['categoryId']));
			}
		}
		$pString = $this->success->text("subcategory", " " . $this->messages->text("misc", "edited") . " ");
		$update['subcategorySubcategory'] = $input['subcategoryEdit'];
		$update['subcategoryCategoryId'] = $input['categoryId'];
		$this->db->formatConditions(array('subcategoryId' => $input['subcategoryEditId']));
		$this->db->update('subcategory', $update);
		return $this->subInit($pString);
	}
// validate input
	private function validateInput($type)
	{
		if($type == 'addCat')
		{
			if(!$input = trim($this->vars['categoryAdd']))
				return FALSE;
		}
		else if($type == 'addSub')
		{
			if(!trim($this->vars['addSubcategory']) || !$this->vars['categoryId'])
				return FALSE;
			$input['addSubcategory'] = trim($this->vars['addSubcategory']);
			$input['categoryId'] = $this->vars['categoryId'];
		}
		else if($type == 'deleteCat')
		{
			if(!$this->vars['categoryIds'])
				return FALSE;
			$input = $this->vars['categoryIds'];
		}
		else if($type == 'deleteCatConfirm')
		{
			if(!$this->vars['categoryIds'])
				return FALSE;
			$input = unserialize(base64_decode($this->vars['categoryIds']));
		}
		else if($type == 'deleteSub')
		{
			if(!$this->vars['subcategoryIds'])
				return FALSE;
			$input = $this->vars['subcategoryIds'];
		}
		else if($type == 'deleteSubConfirm')
		{
			if(!$this->vars['subcategoryIds'])
				return FALSE;
			$input = unserialize(base64_decode($this->vars['subcategoryIds']));
		}
		else if($type == 'editCat')
		{
			if(!trim($this->vars['categoryEdit']) || !$this->vars['categoryEditId'])
				return FALSE;
			$input['text'] = trim($this->vars['categoryEdit']);
			$input['id'] = $this->vars['categoryEditId'];
		}
		else if($type == 'editSub')
		{
			if(!trim($this->vars['subcategoryEdit']) || !array_key_exists('subcategoryEditId', $this->vars) || 
				!array_key_exists('categoryId', $this->vars) || !$this->vars['subcategoryEditId'] || !$this->vars['categoryId'])
				return FALSE;
			$input['subcategoryEdit'] = trim($this->vars['subcategoryEdit']);
			$input['subcategoryEditId'] = $this->vars['subcategoryEditId'];
			$input['categoryId'] = $this->vars['categoryId'];
		}
		return $input;
	}
	private function transferArrow($function)
	{
		$jsonArray = array();
		$jsonArray[] = array(
			'startFunction' => $function,
			);
		$image = $this->ajax->jActionIcon('toBottom', 'onclick', $jsonArray);
		return $image;
	}
}
?>