<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2013
sirfragalot@users.sourceforge.net
**********************************************************************************/
/**
*	QUICKSEARCH class
*
*	Quickly search database
*
*/
class QUICKSEARCH
{
private $db;
private $vars;
private $stmt;
private $errors;
private $messages;
private $common;
private $session;
private $form;
private $table;
private $html;
private $template;
private $keyword;
private $keywords;
private $input;
private $badInput;
private $initCells;
private $parsePhrase;
private $words = '';
private $typeArray;
private $joinResourceId;
private $tAlias = 1;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->stmt = FACTORY_SQLSTATEMENTS::getInstance();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->common = FACTORY_LISTCOMMON::getInstance();
		$this->common->quickSearch = TRUE;
		$this->session = FACTORY_SESSION::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->keyword = FACTORY_KEYWORD::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->parsePhrase = FACTORY_PARSEPHRASE::getInstance();
	}
// display form options
	public function init($error = FALSE, $tableBorder = FALSE, $returnString = FALSE)
	{
///First check, do we have resources?
		if(!$this->common->resourcesExist())
			return;
		include_once("core/modules/help/HELPMESSAGES.php");
		$help = new HELPMESSAGES();
		GLOBALS::setOutputHelp($help->createLink('search'));
		if(!$returnString)
			GLOBALS::setOutputHeading($this->messages->text("heading", "search"));
		$this->session->delVar('mywikindx_PagingStart');
		$this->session->delVar('mywikindx_PagingStartAlpha');
		if(!$tableBorder && $returnString) // FRONT.php display
			$this->session->clearArray('search');
		$pString = $error ? $error : FALSE;
		$pString .= $this->form->formHeader("list_QUICKSEARCH_CORE");
		$pString .= $this->form->hidden("method", "process");
		if($tableBorder)
			$pString .= $this->table->tableStart('generalTable', 1, 0, 4);
		else
			$pString .= $this->table->tableStart(FALSE, 0, 0, 0, FALSE, '50%');
		$pString .= $this->table->trStart();
		$this->radioButtons = FALSE;
		$word = $this->session->issetVar("search_Word") ? 
			htmlspecialchars(stripslashes($this->session->getVar("search_Word")), ENT_QUOTES) : FALSE;
		$hint = $this->html->br() . $this->html->span($this->messages->text("hint", "wordLogic"), 'hint');
		$pString .= $this->table->td($this->form->textInput($this->messages->text("search", "word"), "search_Word", 
			$word, 40) . $hint . '&nbsp;&nbsp;&nbsp;' . $this->form->formSubmit("Search"));
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->form->formEnd();
		if($returnString)
			return $pString; // cf FRONT.php or process() below.
		else
			GLOBALS::buildOutputString($pString);
	}
// Reset the form and clear the session
	public function reset()
	{
		$this->session->clearArray('search');
		$this->init();
	}
// parse the search word(s)
	private function parseWord()
	{
		$this->words = $this->parsePhrase->parse($this->input);
		if(!$this->words)
		{
			$this->template->tpl->assign('resourceListSearchForm', FALSE);
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'init');
		}
	}
// add to SQL conditional statement and add fields to fieldArray for database fields
	private function fieldSql()
	{
		$conditionArray = array();
		$this->joinResourceId = 'id';
// If $this->input['FieldMethod'] == 'OR', we use UNIONS for efficient SQL statements
// As of June 2013, QUICKSEARCH has been simplified to only the textinput. All fields are searched using 'OR'
// customShort
		$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('resourcecustomShort'), $this->words);
		$this->db->formatConditions("($wc)");
		$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_custom', 
			array(array('resourcecustomResourceId' => 'id'))));
// customLong
		$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('resourcecustomLong'), $this->words);
		$this->db->formatConditions("($wc)");
		$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_custom', 
			array(array('resourcecustomResourceId' => 'id'))));
// title
		$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('resourceTitleSort'), $this->words);
		$this->db->formatConditions("($wc)");
		$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource', 
			array(array('resourceId' => 'id'))));
// note
		$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('resourcetextNote'), $this->words);
		$this->db->formatConditions("($wc)");
		$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_text', 
			array(array('resourcetextId' => 'id'))));
// abstract
		$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('resourcetextAbstract'), $this->words);
		$this->db->formatConditions("($wc)");
		$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_text', 
			array(array('resourcetextId' => 'id'))));
// quote
		$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('resourcequotetextText'), $this->words);
		$this->db->leftJoin('resource_quote_text', 'resourcequotetextId', 'resourcequoteId');
		$this->db->formatConditions("($wc)");
		$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_quote', 
			array(array('resourcequoteResourceId' => 'id'))));
// paraphrase
		$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('resourceparaphrasetextText'), $this->words);
		$this->db->leftJoin('resource_paraphrase_text', 'resourceparaphrasetextId', 'resourceparaphraseId');
		$this->db->formatConditions("($wc)");
		$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_paraphrase', 
			array(array('resourceparaphraseResourceId' => 'id'))));
// musing
		$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('resourcemusingtextText'), $this->words);
		$this->db->leftJoin('resource_musing_text', 'resourcemusingtextId', 'resourcemusingId');
		$this->db->formatConditions("($wc)");
		$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_musing', 
			array(array('resourcemusingResourceId' => 'id'))));
// keyword
		$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('keywordKeyword'), $this->words);
		$this->db->leftJoin('keyword', 'keywordId', 'resourcekeywordKeywordId');
		$this->db->formatConditions("($wc)");
// Need this check because resource_keyword tables also stores metadata keywords, not just resource keywords
		$this->db->formatConditions($this->db->formatFields('resourcekeywordResourceId') . ' IS NOT NULL');
		$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_keyword', 
			array(array('resourcekeywordResourceId' => 'id'))));
// creator
		$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('creatorSurname'), $this->words);
		$this->db->formatConditions("($wc)");
		$this->db->leftJoin('creator', 'creatorId', 'resourcecreatorCreatorId');
		$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_creator', 
			array(array('resourcecreatorResourceId' => 'id'))));
// userTag
		$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('usertagsTag'), $this->words);
		$this->db->formatConditions(array('usertagsUserId' => $this->session->getVar('setup_UserId')));
		$this->db->leftJoin('user_tags', 'usertagsId', 'resourceusertagsTagId');
		$this->db->formatConditions("($wc)");
		$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_user_tags', 
			array(array('resourceusertagsId' => 'id'))));
		$this->stmt->unions = $this->db->union($fieldUnions);
	}
/**
* For re-ordering or paging
*/
	public function reprocess()
	{
		$this->input = $this->session->getArray("search");
		if(array_key_exists("search_Order", $this->vars) && $this->vars["search_Order"])
		{
			$this->input['order'] = $this->vars["search_Order"];
			$this->session->setVar('search_Order', $this->input['order']);
			$this->session->setVar('sql_LastOrder', $this->input['order']);
			$this->session->setVar('search_AscDesc', $this->vars['search_AscDesc']);
		}
		$this->process(TRUE);
	}
	public function process($reprocess = FALSE)
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "search"));
		$this->stmt->listMethodAscDesc = 'search_AscDesc';
		$this->stmt->listType = 'search';
		$queryString = 'action=list_QUICKSEARCH_CORE&method=reprocess';
		if(!$reprocess)
			$this->input = $this->checkInput();
		if(array_key_exists('order', $this->input))
			$order = $this->input['order'];
		else
			$order = 'creator';
		$this->input['Partial'] = TRUE;
		$this->template->tpl->assign('resourceListSearchForm', $this->init(FALSE, TRUE, TRUE));
		$this->parseWord();
		$this->fieldSql();
		$sqlTotal = $this->stmt->createTotalResourceSubqueryList(FALSE, FALSE, $order, TRUE, $queryString);
		$this->session->setVar("bookmark_DisplayAdd", TRUE);
		if(array_key_exists('type', $this->vars) && ($this->vars['type'] == 'lastMulti'))
		{
			$this->common->lastMulti('search');
			return;
		}
		$searchTerms = explode(",", $this->session->getVar('search_Highlight'));
		foreach($searchTerms as $term)
			$patterns[] = "/($term)(?=[^>]*(<|$))/i";
		$this->common->patterns = $patterns;
		$this->common->keepHighlight = TRUE;
		if($this->session->getVar('search_DisplayAttachment'))
			$sql = $sqlTotal;
		else
			$sql = $this->stmt->listList(FALSE, FALSE, $order);
		$this->common->display($sql, 'search');
// set the lastMulti session variable for quick return to this process.
		$this->session->setVar('sql_LastMulti', $queryString);
// Turn on the 'add bookmark' menu item
		$this->session->saveState('search');
		$this->session->saveState('sql');
		$this->session->saveState('bookmark');
	}
// write input to session
	private function writeSession()
	{
// First, write all input with 'search_' prefix to session
		foreach($this->vars as $key => $value)
		{
			if(preg_match("/^search_/", $key))
			{
				$key = str_replace('search_', '', $key);
// Is this a multiple select box input?  If so, multiple choices are written to session as 
// comma-delimited string (no spaces).
// Don't write any FALSE or '0' values.
				if(is_array($value))
				{
					if(!$value[0] || ($value[0] == $this->messages->text("misc", "ignore")))
						unset($value[0]);
					$value = implode(",", $value);
				}
				if(!trim($value))
					continue;
				$temp[$key] = trim($value);
			}
		}
		$this->session->clearArray("search");
		$this->session->writeArray($temp, 'search');
	}
// validate user input - method, word and field are required.
// Input comes either from form input or, when paging, from the session.
	private function checkInput()
	{
		$this->writeSession();
		if((array_key_exists("search_Word", $this->vars) && !trim($this->vars["search_Word"])) 
		|| !$this->session->getVar("search_Word"))
		{
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		}
		return $this->session->getArray("search");
	}
}
?>
