<?php
/**
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2013
sirfragalot@users.sourceforge.net
*/
/**
* UPDATEDATABASE
*
* Update the database as required
* 
* @version	1
*
*	@package wikindx4\core\startup
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class UPDATEDATABASE
{
/** object */
private $session;
/** object */
private $config;
/** object */
private $db;
/** array */
private $vars;
/** object */
private $html;
/** object */
private $messages;
/** object */
private $errors;
/**  int */
private $oldTime; 
/** object */
private $utf8;
/** string */
private $stageInterruptMessage = FALSE;
/** string */
private $statusString;
/** string */
private $email = FALSE;
/** integer */
private $numStages;
/** array */
private $tables = array('resource_attachments', 'bibtex_string', 'cache', 'category', 'collection', 'config', 'creator', 
			'custom', 'database_summary', 'import_raw', 'keyword', 'news', 'papers', 'publisher', 'resource', 
			'resource_category', 'resource_creator', 'resource_custom', 'resource_keyword', 
			'resource_misc', 'resource_musing', 'resource_musing_text', 'resource_text', 'resource_page', 
			'resource_paraphrase', 'resource_paraphrase_comment', 'resource_paraphrase_text', 
			'resource_quote', 'resource_quote_comment', 'resource_quote_text', 'resource_summary', 'resource_timestamp', 
			'resource_year', 'tag', 'users', 'user_bibliography', 'user_groups', 'user_register',
			'user_tags', 'resource_user_tags', 'subcategory', 'statistics');

/**
* UPDATEDATABASE
*/
	public function __construct()
	{ 
//print 'START MEMORY: ' . memory_get_peak_usage() / 1000000 . 'MB<P>';
// Turn on error reporting:
		if(version_compare(PHP_VERSION, '5.3.0', '<')) 
			error_reporting(E_ALL & ~E_STRICT);
		else
			error_reporting(E_ALL & ~E_DEPRECATED & ~E_STRICT);
		ini_set('display_errors', 1);
		ini_set('track_errors', 'On');
		$this->db = FACTORY_DB::getInstance();
//		$this->db->query( "SET sql_mode='ONLY_FULL_GROUP_BY'");
		$this->config = FACTORY_CONFIG::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->html = FACTORY_HTML::getInstance();
/*
if(is_array($array = $this->session->getArray('setup')))
{
	print 'SESSION SETUP:<BR>';
	foreach($array as $key => $var)
		print "$key: $var<BR>";
	print '<P>';
}
*/
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->utf8 = FACTORY_UTF8::getInstance();
		$this->vars = GLOBALS::getVars();
/*
print 'VARS:<BR>';
foreach($this->vars as $key => $var)
	print "$key: $var<BR>";
print '<P>';
*/
		$this->oldTime = time();
		$this->checkDatabase();
// Return to default WIKINDX 4.x status
// Avoid grouping errors such as:
// (Mixing of GROUP columns (MIN(),MAX(),COUNT(),...) with no GROUP columns is illegal if there is no GROUP BY clause)
//		$this->db->query("SET sql_mode='TRADITIONAL'");
// Turn off error reporting
		if(version_compare(PHP_VERSION, '5.3.0', '<')) 
			error_reporting(~E_STRICT);
		else
			error_reporting(~E_DEPRECATED & ~E_STRICT);
		ini_set('display_errors', 'Off');
		ini_set('track_errors', 'Off');
		error_reporting(0);
		if(GLOBALS::getOutputString())
			FACTORY_CLOSE::getInstance(); // die;
	}
/**
* Check and print status of update scripts
*
* @param string $stage
*/
	private function checkStatus($stage)
	{
//		print "ELAPSED TIME ($stage): "; print time() - $this->oldTime . " secs.<BR>Database queries: " . GLOBALS::getDbQueries() . '<BR>';
//		print 'MEMORY USED: ' . memory_get_peak_usage() / 1000000 . 'MB<BR>';
		$pString = "MAX EXECUTION TIME: " . ini_get("max_execution_time") .  ' secs' . $this->html->br();
		$pString .= "ELAPSED TIME ($stage): ";
		$pString .= time() - $this->oldTime . " secs.";
		$pString .= $this->html->br(); 
		$pString .= "Database queries: " . GLOBALS::getDbQueries() . $this->html->br();
		$pString .= 'MEMORY LIMIT: ' . ini_get("memory_limit") . $this->html->br();
		$pString .= 'MEMORY USED: ' . memory_get_peak_usage() / 1000000 . 'MB';
		$this->statusString = $pString;
	}
/**
* We know we have a database as, if we've reached this stage, we're able to connect to it.
*
* Here,
* 1/ we check we have tables, if not, populate the database with tables and set defaults in
* database_summary and category table and
* 2/ populate the config table displaying configuration interface if necessary
* If no admins yet exist, ask for admin configuration to force the input of at least one admin username/password.
*/
	private function checkDatabase()
	{
		$array = $this->db->listTables();
// Some users may have one database shared for several different purposes so check for presence of config table
		$table = $this->config->WIKINDX_DB_TABLEPREFIX . 'config';
// 1/
		if(!$array || (sizeof($array) == 0) || 
			((array_search($table, $array) === FALSE) 
			&& (array_search(strtolower($table), $array) === FALSE)))
		{
			$this->db->createSQL();
			$this->setToUtf8();
			$this->populateTables();
			$this->session->setVar("setup_Superadmin", TRUE); // required for gatekeep function in CONFIG.php
		}
		if(array_key_exists('action', $this->vars) && $this->session->getVar('setup_Superadmin') && 
			(($this->vars['action'] == 'continueExecution') || ($this->vars['action'] == 'upgradeDB')))
			$confirm = TRUE;
		else 
			$confirm = FALSE;
		if(!$this->updateDatabase($confirm))
			GLOBALS::buildOutputString($this->error->text("dbError", "updateMySQL"));
// 2/
		$this->checkConfigTable();
	}
/**
* Update the database if required
*
* @param boolean $confirm
* @return boolean
*/
	private function updateDatabase($confirm)
	{
		if(array_search('dbVersion', $this->db->listFields('database_summary'))) // pre v4.0
		{
			$recordset = $this->db->select('database_summary', "dbVersion");
			$dbVersion = $this->db->fetchOne($recordset);
			$recordset = $this->db->select('config', "contactEmail");
			$this->email = $this->db->fetchOne($recordset);
		}
		else
		{
			$recordset = $this->db->select('database_summary', "databasesummaryDbVersion");
			$dbVersion = $this->db->fetchOne($recordset);
			$recordset = $this->db->select('config', "configContactEmail");
			$this->email = $this->db->fetchOne($recordset);
		}
		$this->db->freeSet($recordset);
// WIKINDX 4 cleared out a lot of database upgrade code....
		if($dbVersion < 3.8)
		{
			GLOBALS::buildOutputString("Your WIKINDX database version is $dbVersion.  WIKINDX requires 
				that you first upgrade to WIKINDX v3.8.2 before attempting to upgrade to the latest version.");
			FACTORY_CLOSENOMENU::getInstance(); // die
		}
		if($dbVersion < WIKINDX_DB_VERSION) // v3.8 and up to 4.1
		{
			if(!$confirm)
			{
					$this->confirmUpdateDisplay($dbVersion, WIKINDX_DB_VERSION);
					return FALSE;
			}
			error_reporting(E_ALL & ~E_STRICT);
			if($dbVersion < 4.0) // upgrade to v3.8.2 to v4.2
			{
				$this->numStages = 16;
				$this->upgrade41();
			}
			else if($dbVersion < 4.2) // upgrade v4.1 to v4.2
			{
				$this->numStages = 1;
				$this->stage4_2__1();
			}
			$this->db->query('SET NAMES utf8');
			$this->db->query('SET CHARACTER SET utf8');
			$this->db->update('database_summary', array("databasesummaryDbVersion" => WIKINDX_DB_VERSION));
			$this->db->escapeString = FALSE;
			$this->session->delVar('upgrade_function');
		}
		return TRUE;
	}
/**
* Upgrade database to v4.1.
* Start with stage1();
*/
	private function upgrade41()
	{
		if(array_key_exists('action', $this->vars) && ($this->vars['action'] == 'continueExecution'))
		{
			$function = $this->session->getVar('upgrade_function');
			$this->{$function}();
			return;
		}
		$this->stage4_1__1();
	}
/**
* Stage 1
*/
	private function stage4_1__1()
	{
		$this->session->delVar('stage3UpgradeContinueView');
		$this->session->delVar('stage3UpgradeContinueAttach');
// Set for Latin1
		$this->db->query('SET NAMES latin1');
		$this->db->query('SET CHARACTER SET latin1');
// add temporary indices to speed up the upgrade
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'collection` ADD INDEX(`collectionTitle`)');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'collection` ADD INDEX(`collectionTitleShort`)');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'collection` ADD INDEX(`collectionType`)');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'creator` ADD INDEX(`firstname`)');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'creator` ADD INDEX(`surname`)');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'creator` ADD INDEX(`initials`)');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'creator` ADD INDEX(`prefix`)');
// We no longer use this column.
		$this->db->queryNoError('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'collection` DROP COLUMN `collectionString`');
// new columns
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "resource_misc` ADD `quarantine` 
			VARCHAR(1) NOT NULL DEFAULT 'N'");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "resource_misc` ADD `accessesPeriod` 
			INT(11) NOT NULL DEFAULT '1'");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "config` ADD `quarantine` 
			VARCHAR(1) NOT NULL DEFAULT 'N'");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "config` ADD `noSort` 
			MEDIUMTEXT DEFAULT NULL");
		$value = base64_encode(serialize(array(
'an', 'a', 'the', 'der', 'die', 'das',
'ein', 'eine', 'einer', 'eines', 'le', 'la', 'las',
'il', 'les', 'une', 'un', 'una', 'uno', 'lo', 'los',
'i', 'gli', 'de', 'het', 'um', 'uma', 'o', 'os', 'as',
)));
		$this->db->update('config', array('noSort' => $value));
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "config` ADD `searchFilter` 
			MEDIUMTEXT DEFAULT NULL");
		$value = base64_encode(serialize(array(
'an', 'a', 'the', 'and', 'to')));
		$this->db->update('config', array('searchFilter' => $value));
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "config` ADD `listlink` 
			VARCHAR(1) NOT NULL DEFAULT 'N'");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "config` ADD `emailStatistics` 
			VARCHAR(1) NOT NULL DEFAULT 'N'");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "config` 
			ADD COLUMN `statisticsCompiled` DATETIME DEFAULT '00/00/00 00:00:00'");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "config` 
			ADD COLUMN `metadataAllow` VARCHAR(1) DEFAULT 'Y'");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "config` 
			ADD COLUMN `metadataUserOnly` VARCHAR(1) DEFAULT 'N'");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "config` 
			ADD COLUMN `denyReadOnly` VARCHAR(1) DEFAULT 'N'");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "config` 
			ADD COLUMN `readOnlyAccess` VARCHAR(1) DEFAULT 'Y'");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "config` 
			ADD COLUMN `originatorEditOnly` VARCHAR(1) DEFAULT 'N'");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "config` 
			ADD COLUMN `globalEdit` VARCHAR(1) DEFAULT 'N'");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "config` 
			ADD COLUMN `errorReport` VARCHAR(1) DEFAULT 'N'");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "config` 
			ADD COLUMN `printSql` VARCHAR(1) DEFAULT 'N'");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "config` 
			ADD COLUMN `captchaPublicKey` VARCHAR(255) DEFAULT NULL");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "config` 
			ADD COLUMN `captchaPrivateKey` VARCHAR(255) DEFAULT NULL");
		$date = date("y/m") . '/01 00:00:00'; // i.e. 1st of this month
		$this->db->updateTimestamp('config', array('statisticsCompiled' => "'" . $date . "'"));
		$this->upgrade40Nulls();
		$this->swapNumPages();
		$this->checkStatus('stage4_1__1');
		$this->pauseExecution('stage4_1__2', 'stage4_1__1');
	}
/**
* Stage 2
*/
	private function stage4_1__2()
	{
// Set for Latin1
		$this->db->query('SET NAMES latin1');
		$this->db->query('SET CHARACTER SET latin1');
		$this->upgrade40CorrectBackSlash();
		$this->upgrade40CreateUsertags();
		$this->upgrade40CreateSubcategory();
		$this->upgrade40CreateResourceText();
		$this->checkStatus('stage4_1__2');
		$this->pauseExecution('stage4_1__3', 'stage4_1__2');
	}
/**
* Stage 3
*/
	private function stage4_1__3()
	{
// Set for Latin1
		$this->db->query('SET NAMES latin1');
		$this->db->query('SET CHARACTER SET latin1');
		$this->upgrade40CreateResourceAttachments();
		$this->checkStatus('stage4_1__3');
		$this->pauseExecution('stage4_1__4', 'stage4_1__3');
	}
/**
* stage 4
*/
	private function stage4_1__4()
	{
// Set for Latin1
		$this->db->query('SET NAMES latin1');
		$this->db->query('SET CHARACTER SET latin1');
		$this->upgrade40RemoveDupCreators();
		$this->checkStatus('stage4_1__4');
		$this->pauseExecution('stage4_1__5', 'stage4_1__4');
	}
/**
* stage 5
*/
	private function stage4_1__5()
	{
// Set for Latin1
		$this->db->query('SET NAMES latin1');
		$this->db->query('SET CHARACTER SET latin1');
		$this->upgrade40RemoveDupCollections();
		$this->checkStatus('stage4_1__5');
		$this->pauseExecution('stage4_1__6', 'stage4_1__5');
	}
/**
* stage 6
*/
	private function stage4_1__6()
	{
// Set for Latin1
		$this->db->query('SET NAMES latin1');
		$this->db->query('SET CHARACTER SET latin1');
// Drop this table left over from v3.x
		$this->db->query('DROP TABLE IF EXISTS `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'notify`');
// Drop these columns
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'config` DROP COLUMN `kwBibliography`');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'users` DROP COLUMN `kwBibliography`');
		$this->upgrade40Rename();
		$this->upgrade40charToBin();
		$this->checkStatus('stage4_1__6');
		$this->pauseExecution('stage4_1__7', 'stage4_1__6');
	}
/**
* stage 7
*/
	private function stage4_1__7()
	{
// Set for Latin1
		$this->db->query('SET NAMES latin1');
		$this->db->query('SET CHARACTER SET latin1');
// Set database defaults to UTF8
		$this->db->query("ALTER DATABASE `" . $this->config->WIKINDX_DB . 
			"` DEFAULT CHARACTER SET utf8 COLLATE utf8_unicode_ci");
		$this->upgrade40Tables();
// Initially set all users to default settings
		$this->db->update('users', 
			array('usersStyle' => 'APA', 'usersLanguage' => 'en', 'usersTemplate' => 'default', 'usersCookie' => 'N'));
// Same for read-only users in config table
		$this->db->update('config', 
			array('configStyle' => 'APA', 'configLanguage' => 'en', 'configTemplate' => 'default'));
// Set for UTF8
		$this->db->query('SET NAMES utf8');
		$this->db->query('SET CHARACTER SET utf8');
		$this->checkStatus('stage4_1__7');
		$this->pauseExecution('stage4_1__8', 'stage4_1__7');
	}
/**
* stage 8
*/
	private function stage4_1__8()
	{
		$this->rewriteResourceCreator();
		$this->checkStatus('stage4_1__8');
		$this->pauseExecution('stage4_1__9', 'stage4_1__8');
	}
/**
* stage 9
*/
	private function stage4_1__9()
	{
		$this->rewriteResourceSummary();
		$this->checkStatus('stage4_1__9');
		$this->pauseExecution('stage4_1__10', 'stage4_1__9');
	}
/**
* stage 10
*/
	private function stage4_1__10()
	{
		$this->rewriteResourceKeyword();
		$this->checkStatus('stage4_1__10');
		$this->pauseExecution('stage4_1__11', 'stage4_1__10');
	}
/**
* stage 11
*/
	private function stage4_1__11()
	{
		$this->rewriteResourceCategory();
		$this->checkStatus('stage4_1__11');
		$this->pauseExecution('stage4_1__12', 'stage4_1__11');
	}
/**
* stage 12
*/
	private function stage4_1__12()
	{
		$this->rewriteBibliographiesGroups();
		$this->checkStatus('stage4_1__12');
		$this->pauseExecution('stage4_1__13', 'stage4_1__12');
	}
/**
* stage 13
*
* convert bbcode to html for use with tinymce and correct transTitle errors
*/
	private function stage4_1__13()
	{
		$bbcode = FACTORY_BBCODE::getInstance();
		$html = FACTORY_HTML::getInstance();
		$string = $this->db->fetchOne($this->db->select('config', 'configDescription'));
		$string = $bbcode->codeToHtml($string);
		$this->db->update('config', array('configDescription' => $string));
// correct transTitle errors
		$resultset = $this->db->select('resource', array('resourceId', 'resourceTransTitle', 'resourceTransSubtitle'));
		while($row = $this->db->fetchRow($resultset))
		{
			$nulls = array();
			if(($row['resourceTransTitle'] == '') || ($row['resourceTransTitle'] == '(no title)'))
				$nulls[] = 'resourceTransTitle';
			if($row['resourceTransSubtitle'] == '')
				$nulls[] = 'resourceTransSubtitle';
			if(!empty($nulls))
			{
				$this->db->formatConditions(array('resourceId' => $row['resourceId']));
				$this->db->updateNull('resource', $nulls);
			}
		}
		$this->db->freeSet($resultset);
// titles and subtitles
		$this->db->formatConditions($this->db->formatFields('resourceTitle') . $this->db->like('%', '[', '%') . 
			$this->db->or . $this->db->formatFields('resourceSubtitle') . $this->db->like('%', '[', '%'));
		$resultset = $this->db->select('resource', array('resourceId', 'resourceTitle', 'resourceSubtitle', 
			'resourceTransTitle', 'resourceTransSubtitle'));
		while($row = $this->db->fetchRow($resultset))
		{
			$updateArray = array();
			if($row['resourceTitle'])
			{
				$string = $bbcode->codeToHtml($row['resourceTitle']);
				$updateArray['resourceTitle'] = $string;
			}
			if($row['resourceSubtitle'])
			{
				$string = $bbcode->codeToHtml($row['resourceSubtitle']);
				$updateArray['resourceSubtitle'] = $string;
			}
			if($row['resourceTransTitle'])
			{
				$string = $bbcode->codeToHtml($row['resourceTransTitle']);
				$updateArray['resourceTransTitle'] = $string;
			}
			if($row['resourceTransSubtitle'])
			{
				$string = $bbcode->codeToHtml($row['resourceTransSubtitle']);
				$updateArray['resourceTransSubtitle'] = $string;
			}
			if(!empty($updateArray))
			{
				$this->db->formatConditions(array('resourceId' => $row['resourceId']));
				$this->db->update('resource', $updateArray);
			}
		}
		$this->db->freeSet($resultset);
// Collection titles
		$this->db->formatConditions($this->db->formatFields('collectionTitle') . $this->db->like('%', '[', '%'));
		$resultset = $this->db->select('collection', array('collectionId', 'collectionTitle'));
		while($row = $this->db->fetchRow($resultset))
		{
			$updateArray = array();
			if($row['collectionTitle'])
			{
				$string = $bbcode->codeToHtml($row['collectionTitle']);
				$updateArray['collectionTitle'] = $string;
			}
			if(!empty($updateArray))
			{
				$this->db->formatConditions(array('collectionId' => $row['collectionId']));
				$this->db->update('collection', $updateArray);
			}
		}
		$this->db->freeSet($resultset);
// notes and abstracts
		$this->db->formatConditions($this->db->formatFields('resourcetextNote') . $this->db->like('%', '[', '%') . 
			$this->db->or . $this->db->formatFields('resourcetextAbstract') . $this->db->like('%', '[', '%'));
		$resultset = $this->db->select('resource_text', array('resourcetextId', 'resourcetextNote', 'resourcetextAbstract'));
		while($row = $this->db->fetchRow($resultset))
		{
			$updateArray = array();
			if($row['resourcetextNote'])
			{
				$string = $bbcode->codeToHtml($row['resourcetextNote']);
				$updateArray['resourcetextNote'] = $string;
			}
			if($row['resourcetextAbstract'])
			{
				$string = $bbcode->codeToHtml($row['resourcetextAbstract']);
				$updateArray['resourcetextAbstract'] = $string;
			}
			if(!empty($updateArray))
			{
				$this->db->formatConditions(array('resourcetextId' => $row['resourcetextId']));
				$this->db->update('resource_text', $updateArray);
			}
		}
		$this->db->freeSet($resultset);
// Custom large fields -- don't filter results because all fields need newlines converting to HTML
//		$this->db->formatConditions($this->db->formatFields('resourcecustomLong') . $this->db->like('%', '[', '%'));
		$resultset = $this->db->select('resource_custom', array('resourcecustomId', 'resourcecustomLong'));
		while($row = $this->db->fetchRow($resultset))
		{
			$updateArray = array();
			if($row['resourcecustomLong'])
			{
				$string = $bbcode->codeToHtml($row['resourcecustomLong']);
				$updateArray['resourcecustomLong'] = $html->nlToHtml($string, TRUE);
			}
			if(!empty($updateArray))
			{
				$this->db->formatConditions(array('resourcecustomId' => $row['resourcecustomId']));
				$this->db->update('resource_custom', $updateArray);
			}
		}
		$this->db->freeSet($resultset);
// User bibliography
//		$this->db->formatConditions($this->db->formatFields('userbibliographyDescription') . $this->db->like('%', '[', '%'));
		$resultset = $this->db->select('user_bibliography', array('userbibliographyId', 'userbibliographyDescription'));
		while($row = $this->db->fetchRow($resultset))
		{
			$updateArray = array();
			if($row['userbibliographyDescription'])
			{
				$string = $bbcode->codeToHtml($row['userbibliographyDescription']);
				$updateArray['userbibliographyDescription'] = $html->nlToHtml($string, TRUE);
			}
			if(!empty($updateArray))
			{
				$this->db->formatConditions(array('userbibliographyId' => $row['userbibliographyId']));
				$this->db->update('user_bibliography', $updateArray);
			}
		}
		$this->db->freeSet($resultset);
// News
//		$this->db->formatConditions($this->db->formatFields('newsNews') . $this->db->like('%', '[', '%'));
		$resultset = $this->db->select('news', array('newsId', 'newsNews'));
		while($row = $this->db->fetchRow($resultset))
		{
			$updateArray = array();
			if($row['newsNews'])
			{
				$string = $bbcode->codeToHtml($row['newsNews']);
				$updateArray['newsNews'] = $html->nlToHtml($string, TRUE);
			}
			if(!empty($updateArray))
			{
				$this->db->formatConditions(array('newsId' => $row['newsId']));
				$this->db->update('news', $updateArray);
			}
		}
		$this->db->freeSet($resultset);
// Musings
//		$this->db->formatConditions($this->db->formatFields('resourcemusingtextText') . $this->db->like('%', '[', '%'));
		$resultset = $this->db->select('resource_musing_text', array('resourcemusingtextId', 'resourcemusingtextText'));
		while($row = $this->db->fetchRow($resultset))
		{
			$updateArray = array();
			if($row['resourcemusingtextText'])
			{
				$string = $bbcode->codeToHtml($row['resourcemusingtextText']);
				$updateArray['resourcemusingtextText'] = $html->nlToHtml($string, TRUE);
			}
			if(!empty($updateArray))
			{
				$this->db->formatConditions(array('resourcemusingtextId' => $row['resourcemusingtextId']));
				$this->db->update('resource_musing_text', $updateArray);
			}
		}
		$this->db->freeSet($resultset);
// Quotes
//		$this->db->formatConditions($this->db->formatFields('resourcequotetextText') . $this->db->like('%', '[', '%'));
		$resultset = $this->db->select('resource_quote_text', array('resourcequotetextId', 'resourcequotetextText'));
		while($row = $this->db->fetchRow($resultset))
		{
			$updateArray = array();
			if($row['resourcequotetextText'])
			{
				$string = $bbcode->codeToHtml($row['resourcequotetextText']);
				$updateArray['resourcequotetextText'] = $html->nlToHtml($string, TRUE);
			}
			if(!empty($updateArray))
			{
				$this->db->formatConditions(array('resourcequotetextId' => $row['resourcequotetextId']));
				$this->db->update('resource_quote_text', $updateArray);
			}
		}
		$this->db->freeSet($resultset);
// Quote comments
//		$this->db->formatConditions($this->db->formatFields('resourcequotecommentComment') . $this->db->like('%', '[', '%'));
		$resultset = $this->db->select('resource_quote_comment', array('resourcequotecommentId', 'resourcequotecommentComment'));
		while($row = $this->db->fetchRow($resultset))
		{
			$updateArray = array();
			if($row['resourcequotecommentComment'])
			{
				$string = $bbcode->codeToHtml($row['resourcequotecommentComment']);
				$updateArray['resourcequotecommentComment'] = $html->nlToHtml($string, TRUE);
			}
			if(!empty($updateArray))
			{
				$this->db->formatConditions(array('resourcequotecommentId' => $row['resourcequotecommentId']));
				$this->db->update('resource_quote_comment', $updateArray);
			}
		}
		$this->db->freeSet($resultset);
// Paraphrases
//		$this->db->formatConditions($this->db->formatFields('resourceparaphrasetextText') . $this->db->like('%', '[', '%'));
		$resultset = $this->db->select('resource_paraphrase_text', array('resourceparaphrasetextId', 'resourceparaphrasetextText'));
		while($row = $this->db->fetchRow($resultset))
		{
			$updateArray = array();
			if($row['resourceparaphrasetextText'])
			{
				$string = $bbcode->codeToHtml($row['resourceparaphrasetextText']);
				$updateArray['resourceparaphrasetextText'] = $html->nlToHtml($string, TRUE);
			}
			if(!empty($updateArray))
			{
				$this->db->formatConditions(array('resourceparaphrasetextId' => $row['resourceparaphrasetextId']));
				$this->db->update('resource_paraphrase_text', $updateArray);
			}
		}
		$this->db->freeSet($resultset);
// Paraphrase comments
//		$this->db->formatConditions($this->db->formatFields('resourceparaphrasecommentComment') . $this->db->like('%', '[', '%'));
		$resultset = $this->db->select('resource_paraphrase_comment', array('resourceparaphrasecommentId', 'resourceparaphrasecommentComment'));
		while($row = $this->db->fetchRow($resultset))
		{
			$updateArray = array();
			if($row['resourceparaphrasecommentComment'])
			{
				$string = $bbcode->codeToHtml($row['resourceparaphrasecommentComment']);
				$updateArray['resourceparaphrasecommentComment'] = $html->nlToHtml($string, TRUE);
			}
			if(!empty($updateArray))
			{
				$this->db->formatConditions(array('resourceparaphrasecommentId' => $row['resourceparaphrasecommentId']));
				$this->db->update('resource_paraphrase_comment', $updateArray);
			}
		}
		$this->db->freeSet($resultset);
		$this->checkStatus('stage4_1__13');
		$this->pauseExecution('stage4_1__14', 'stage4_1__13');
	}
/**
* stage 14
*/	
	private function stage4_1__14()
	{
		$this->addMissingRows();
// Session setup required in penultimate stage
		$this->session->setVar("setup_UserId", 1);
		$user = FACTORY_USER::getInstance();
		$user->writeSessionPreferences(FALSE, 'config', TRUE);
		$this->checkStatus('stage4_1__14');
		$this->pauseExecution('stage4_1__15', 'stage4_1__14');
	}
/**
* stage 15
*/	
	private function stage4_1__15()
	{
		$this->upgradeResourceTitle();
// Create new language tables
		$this->db->query('CREATE TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'language` 
			(`languageId` int(11) NOT NULL AUTO_INCREMENT,
			`languageLanguage` varchar(255) DEFAULT NULL,
			PRIMARY KEY (`languageId`)) 
			ENGINE=MyISAM CHARSET=utf8
			');
		$this->db->query('CREATE TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource_language` 
			(`resourcelanguageId` int(11) NOT NULL AUTO_INCREMENT,
			`resourcelanguageResourceId` int(11),
			`resourcelanguageLanguageId` int(11),
			PRIMARY KEY (`resourcelanguageId`)) 
			ENGINE=MyISAM CHARSET=utf8
			');
// Drop old indexes
		$this->db->queryNoError('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource_timestamp` DROP INDEX `timestamp`');
		$this->db->queryNoError('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource_timestamp` DROP INDEX `timestampAdd`');
		$this->db->queryNoError('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource` DROP INDEX `type`');
		$this->db->queryNoError('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'publisher` DROP INDEX `publisherName`');
		$this->db->queryNoError('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource_category` DROP INDEX `categories`');
		$this->db->queryNoError('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'keyword` DROP INDEX `keyword`');
		$this->db->queryNoError('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'category` DROP INDEX `category`');
		$this->db->queryNoError('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'creator` DROP INDEX `firstname`');
		$this->db->queryNoError('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'creator` DROP INDEX `surname`');
		$this->db->queryNoError('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource` DROP INDEX `title`');
		$this->db->queryNoError('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'statistics` DROP INDEX `resourceId`');
		$this->db->queryNoError('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'collection` DROP INDEX `collectionTitle`');
		$this->db->queryNoError('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'collection` DROP INDEX `collectionTitleShort`');
		$this->db->queryNoError('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'collection` DROP INDEX `collectionType`');
		$this->db->queryNoError('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'creator` DROP INDEX `firstname`');
		$this->db->queryNoError('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'creator` DROP INDEX `surname`');
		$this->db->queryNoError('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'creator` DROP INDEX `initials`');
		$this->db->queryNoError('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'creator` DROP INDEX `prefix`');
// Add new indices
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource_language` ADD INDEX(`resourcelanguageResourceId`)');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource_language` ADD INDEX(`resourcelanguageLanguageId`)');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource_year` ADD INDEX(`resourceyearYear1`)');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource` ADD INDEX(`resourceType`)');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource_timestamp` 
			ADD INDEX(`resourcetimestampTimestampAdd`)');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource_timestamp` 
			ADD INDEX(`resourcetimestampTimestamp`)');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
			'resource_attachments` ADD INDEX(`resourceattachmentsResourceId`)');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource_user_tags` 
			ADD INDEX(`resourceusertagsResourceId`)');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource` ADD INDEX(`resourceTitleSort`)');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'collection` ADD INDEX(`collectionTitle`)');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'creator` ADD INDEX(`creatorSurname`)');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'keyword` ADD INDEX(`keywordKeyword`)');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'category` ADD INDEX(`categoryCategory`)');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'publisher` ADD INDEX(`publisherName`)');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource_misc` ADD INDEX(`resourcemiscCollection`)');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource_misc` ADD INDEX(`resourcemiscPublisher`)');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource_custom` ADD INDEX(`resourcecustomCustomId`)');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'statistics` ADD INDEX(`statisticsResourceId`)');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'statistics` ADD INDEX(`statisticsAttachmentId`)');
// Add or alter columns
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'cache` 
			ADD COLUMN `cacheKeywords` MEDIUMTEXT DEFAULT NULL');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'creator` 
			ADD COLUMN `creatorSameAs` INT(11)');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'creator` ADD INDEX(`creatorSameAs`)');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'users` 
			ADD COLUMN `usersIsCreator` INT(11)');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "users` 
			ADD COLUMN `usersListlink` VARCHAR(1) DEFAULT 'N'");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'users` 
			ADD COLUMN `usersDepartment` VARCHAR(255)');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "users` 
			ADD COLUMN `usersTemplateMenu` INT(11) DEFAULT  '0'");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "resource_attachments` 
			ADD COLUMN `resourceattachmentsEmbargo` VARCHAR(1) DEFAULT 'N'");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "resource_attachments` 
			ADD COLUMN `resourceattachmentsEmbargoUntil` DATETIME DEFAULT '00/00/00 00:00:00'");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'users` 
			ADD COLUMN `usersInstitution` VARCHAR(255)');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource_misc` 
			DROP COLUMN `resourcemiscAttachDownloads`');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "resource_quote_comment` CHANGE 
			`resourcequotecommentPrivate` `resourcequotecommentPrivate` VARCHAR(255) DEFAULT 'N'");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "resource_paraphrase_comment` CHANGE 
			`resourceparaphrasecommentPrivate` `resourceparaphrasecommentPrivate` VARCHAR(255) DEFAULT 'N'");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "resource_musing_text` CHANGE 
			`resourcemusingtextPrivate` `resourcemusingtextPrivate` VARCHAR(255) DEFAULT 'N'");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "users` CHANGE 
			`usersNotify` `usersNotify` VARCHAR(1) DEFAULT 'N'");
		$this->recreate40Cache();
		ini_set('memory_limit', $this->config->WIKINDX_MEMORY_LIMIT);
		if(array_key_exists('fixUTF8', $this->vars))
		{
			$this->db->escapeString = TRUE;
			$this->fixUTF8();
			$this->db->escapeString = FALSE;
		}
		$this->checkStatus('stage4_1__15');
		$this->pauseExecution('stage4_2__1', 'stage4_1__15');
	}
/**
* Upgrade 4.1 to 4.2 -- Stage 1
*/
	private function stage4_2__1()
	{
// text types greater than VARCHAR cannot have an index in MySQL
		$this->db->queryNoError('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource` DROP INDEX `resourceTitleSort`');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "resource` CHANGE 
			`resourceTitle` `resourceTitle` TEXT");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "resource` CHANGE 
			`resourceSubtitle` `resourceSubtitle` TEXT DEFAULT NULL");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "resource` CHANGE 
			`resourceTransTitle` `resourceTransTitle` TEXT");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "resource` CHANGE 
			`resourceTransSubtitle` `resourceTransSubtitle` TEXT DEFAULT NULL");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "resource` CHANGE 
			`resourceTitleSort` `resourceTitleSort` TEXT");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "resource` CHANGE 
			`resourceTransNoSort` `resourceTransNoSort` TEXT DEFAULT NULL");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "resource` CHANGE 
			`resourceNoSort` `resourceNoSort` TEXT");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "users` CHANGE 
			`usersUserSession` `usersUserSession` LONGTEXT");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "cache` CHANGE 
			`cacheResourceCreators` `cacheResourceCreators` LONGTEXT");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "cache` CHANGE 
			`cacheMetadataCreators` `cacheMetadataCreators` LONGTEXT");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "cache` CHANGE 
			`cacheKeywords` `cacheKeywords` LONGTEXT");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "cache` CHANGE 
			`cacheResourceKeywords` `cacheResourceKeywords` LONGTEXT");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "cache` CHANGE 
			`cacheMetadataKeywords` `cacheMetadataKeywords` LONGTEXT");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "cache` CHANGE 
			`cacheQuoteKeywords` `cacheQuoteKeywords` LONGTEXT");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "cache` CHANGE 
			`cacheParaphraseKeywords` `cacheParaphraseKeywords` LONGTEXT");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "cache` CHANGE 
			`cacheMusingKeywords` `cacheMusingKeywords` LONGTEXT");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "cache` CHANGE 
			`cacheResourcePublishers` `cacheResourcePublishers` LONGTEXT");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "cache` CHANGE 
			`cacheMetadataPublishers` `cacheMetadataPublishers` LONGTEXT");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "cache` CHANGE 
			`cacheConferenceOrganisers` `cacheConferenceOrganisers` LONGTEXT");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "cache` CHANGE 
			`cacheResourceCollections` `cacheResourceCollections` LONGTEXT");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "cache` CHANGE 
			`cacheMetadataCollections` `cacheMetadataCollections` LONGTEXT");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "cache` CHANGE 
			`cacheResourceCollectionTitles` `cacheResourceCollectionTitles` LONGTEXT");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . "cache` CHANGE 
			`cacheResourceCollectionShorts` `cacheResourceCollectionShorts` LONGTEXT");
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource_attachments` 
			ADD COLUMN `resourceattachmentsDescription` TEXT DEFAULT NULL');
// v4.x code before 4.2 incorrectly named statistics field
		$fields = $this->db->listFields('statistics');
		if(array_search('statistics', $fields) !== FALSE)
			$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'statistics` 
				CHANGE `statistics` `statisticsStatistics` TEXT DEFAULT NULL');
		$this->checkStatus('stage4_2__1');
	}
/**
* Fix bad UTF8
*/
	private function fixUTF8()
	{
		include_once("core/utf8/encoding.php");
		$prefix = $this->config->WIKINDX_DB_TABLEPREFIX;
		$tables = $this->db->listTables();
		foreach($tables as $table)
		{
			$discards = array();
			$table = preg_replace("/^$prefix/i", '', $table);
// The following only have timestamps, numeric values or preset values such as 'N', 'Y' etc.
			if(($table == 'cache') || ($table == 'database_summary') || ($table == 'resource_category') || 
				 ($table == 'resource_keyword') || ($table == 'resource_language') || ($table == 'resource_misc') || 
				  ($table == 'resource_summary') || ($table == 'resource_timestamp') || ($table == 'statistics') ||
				  ($table == 'user_bibliography_resource') || ($table == 'user_groups_users'))
				continue;
			$convType = 'lightFixutf8';
// Need to discard datetime types from conversion
			$resultset = $this->db->query($this->db->selectNoExecute($table, '*'));
			$fInfo = mysqli_fetch_fields($resultset); 
		   	foreach ($fInfo as $val)
		   	{
		   		if($val->type <= 16) // Numeric + date/timestamp types
		   			$discards[] = $val->name;
			}
			$resultset = $this->db->select($table, '*');
			while($row = $this->db->fetchRow($resultset))
			{
				if(($table == 'users') && ($field = 'usersPassword'))
					continue;
				$id = str_replace('_', '', $table) . 'Id';
				$updateArray = array();
				foreach($row as $field => $value)
				{
					if(array_search($field, $discards) !== FALSE)
						continue;
					if(!$value || is_numeric($value))
						continue;
					$value = stripslashes($value);
					$original = $value;
// Convert &#xxx; characters
//					if(mb_detect_encoding($value) != 'UTF-8')
//						$value = $this->utf8->encodeUtf8($value);
//					else
//						$value = utf8_decode($value);
//						$value = $this->utf8->decodeUtf8($value);
					if($convType == 'lightFixutf8')
					{
						$value = Encoding::toUTF8($value);
						if($original != $value)
							$updateArray[$field] = $value;
					}
					else if($convType == 'toughFixutf8')
					{
						$value = Encoding::fixUTF8($value);
						if($original != $value)
							$updateArray[$field] = $value;
					}
				}
				if(empty($updateArray))
					continue;
				if(($table != 'config'))
					$this->db->formatConditions(array($id => $row[$id]));
				$this->db->update($table, $updateArray);
			}
			$this->db->freeSet($resultset);
		}
		$this->db->updateNull('cache', 'cacheResourceCreators');
		$this->db->updateNull('cache', 'cacheMetadataCreators');
		$this->db->updateNull('cache', 'cacheKeywords');
		$this->db->updateNull('cache', 'cacheResourceKeywords');
		$this->db->updateNull('cache', 'cacheMetadataKeywords');
		$this->db->updateNull('cache', 'cacheQuoteKeywords');
		$this->db->updateNull('cache', 'cacheParaphraseKeywords');
		$this->db->updateNull('cache', 'cacheMusingKeywords');
		$this->db->updateNull('cache', 'cacheResourcePublishers');
		$this->db->updateNull('cache', 'cacheMetadataPublishers');
		$this->db->updateNull('cache', 'cacheConferenceOrganisers');
		$this->db->updateNull('cache', 'cacheResourceCollections');
		$this->db->updateNull('cache', 'cacheMetadataCollections');
		$this->db->updateNull('cache', 'cacheResourceCollectionTitles');
		$this->db->updateNull('cache', 'cacheResourceCollectionShorts');
	}
/**
* v4.1 switches the numPages data for books and theses from resourceField6 to resourcemiscField6
*/
	private function swapNumPages()
	{
		$resultset = $this->db->query(
			'SELECT `id`, `field6` FROM `' . 
			$this->config->WIKINDX_DB_TABLEPREFIX . 'resource`' .  
			"WHERE (`type` = 'book' 
			OR 
			`type` = 'thesis') 
			AND 
			`field6` IS NOT NULL 
			");
		while($row = $this->db->fetchRow($resultset))
		{
			$this->db->formatConditions(array('id' => $row['id']));
			$this->db->update('resource_misc', array('miscField6' => $row['field6']));
			$this->db->formatConditions(array('id' => $row['id']));
			$this->db->updateNull('resource', array('field6'));
		}
		$this->db->freeSet($resultset);
	}
/**
* To get total counts right in list operations, certain tables, having their ID columns the same as resource.resourceId, 
* must have the same number of rows as the resource table.
*/
	private function addMissingRows()
	{
// resource_year
		$resultset = $this->db->query('SELECT `resourceId` FROM `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource` 
			WHERE `resourceId` NOT IN (SELECT `resourceyearId` 
			FROM `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource_year`)');
		while($row = $this->db->fetchRow($resultset))
			$this->db->insert('resource_year', 'resourceyearId', $row['resourceId']);
		$this->db->freeSet($resultset);
// resource_misc
		$resultset = $this->db->query('SELECT `resourceId` FROM `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource` 
			WHERE `resourceId` NOT IN (SELECT `resourcemiscId` 
			FROM `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource_misc`)');
		while($row = $this->db->fetchRow($resultset))
			$this->db->insert('resource_misc', 'resourcemiscId', $row['resourceId']);
		$this->db->freeSet($resultset);
// resource_timestamp
		$resultset = $this->db->query('SELECT `resourceId` FROM `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource` 
			WHERE `resourceId` NOT IN (SELECT `resourcetimestampId` 
			FROM `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource_timestamp`)');
		while($row = $this->db->fetchRow($resultset))
			$this->db->insert('resource_timestamp', 'resourcetimestampId', $row['resourceId']);
		$this->db->freeSet($resultset);
	}
/**
* Fix quote, paraphrase, musing counts per resource.
*/
	private function rewriteResourceSummary()
	{
		$ids = array();
// quotes
		$this->db->groupBy('resourcequoteResourceId');
		$resultset = $this->db->query('SELECT `resourcequoteResourceId`, COUNT(`resourcequoteResourceId`) AS `count` FROM ' . 
			$this->config->WIKINDX_DB_TABLEPREFIX . 'resource_quote');
		while($row = $this->db->fetchRow($resultset))
			$ids[$row['resourcequoteResourceId']]['resourcesummaryQuotes'] = $row['count'];
		$this->db->freeSet($resultset);
// paraphrases
		$this->db->groupBy('resourceparaphraseResourceId');
		$resultset = $this->db->query('SELECT `resourceparaphraseResourceId`, COUNT(`resourceparaphraseResourceId`) AS `count` FROM ' . 
			$this->config->WIKINDX_DB_TABLEPREFIX . 'resource_paraphrase');
		while($row = $this->db->fetchRow($resultset))
			$ids[$row['resourceparaphraseResourceId']]['resourcesummaryParaphrases'] = $row['count'];
		$this->db->freeSet($resultset);
// musings
		$this->db->groupBy('resourcemusingResourceId');
		$resultset = $this->db->query('SELECT `resourcemusingResourceId`, COUNT(`resourcemusingResourceId`) AS `count` FROM ' . 
			$this->config->WIKINDX_DB_TABLEPREFIX . 'resource_musing');
		while($row = $this->db->fetchRow($resultset))
			$ids[$row['resourcemusingResourceId']]['resourcesummaryMusings'] = $row['count'];
		$this->db->freeSet($resultset);
		foreach($ids as $id => $fieldArray)
		{
			$update = $nulls = array();
			foreach($fieldArray as $field => $count)
				$update[$field] = $count;
			if(!array_key_exists('resourcesummaryQuotes', $fieldArray))
				$nulls[] = 'resourcesummaryQuotes';
			if(!array_key_exists('resourcesummaryParaphrases', $fieldArray))
				$nulls[] = 'resourcesummaryParaphrases';
			if(!array_key_exists('resourcesummaryMusings', $fieldArray))
				$nulls[] = 'resourcesummaryMusings';

			$this->db->formatConditions(array('resourcesummaryId' => $id));
			$this->db->update('resource_summary', $update);
			if(!empty($nulls))
			{
				$this->db->formatConditions(array('resourcesummaryId' => $id));
				$this->db->updateNull('resource_summary', $nulls);
			}
		}
	}
/**
* Create a new column on the resource table that holds the title-subtitle without {}, ' or " in order to facilitate fast sorting
*/
	private function upgradeResourceTitle()
	{
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource` 
			ADD COLUMN `resourceTitleSort` VARCHAR(255) DEFAULT NULL
			');
		$resultset = $this->db->query('SELECT ' . $this->db->replace($this->db->replace($this->db->replace($this->db->replace('resourceTitle', 
			'{', ''), '}', '', FALSE), '"', '', FALSE), '\\\'', '', FALSE) . " AS 't', " . 
			$this->db->replace($this->db->replace($this->db->replace($this->db->replace('resourceSubtitle', 
			'{', ''), '}', '', FALSE), '"', '', FALSE), '\\\'', '', FALSE) . " AS 's', `resourceId` FROM " .  
			$this->config->WIKINDX_DB_TABLEPREFIX . 'resource');
		while($row = $this->db->fetchRow($resultset))
		{
			$title = '';
			if($row['s'])
				$title = $row['t'] . ' ' . $row['s'];
			else
				$title = $row['t'];
			$this->db->formatConditions(array('resourceId' => $row['resourceId']));
			$this->db->update('resource', array('resourceTitleSort' => $title));
		}
		$this->db->freeSet($resultset);
	}
/**
* Rewrite resource_creator tables
*
* In v4, we no longer use MySQL's FIND_IN_SET and therefore no longer want comma-delimited field values such as creator1, creator2 etc. in 
* the resource_creator table.  Write a new resource_creator table that has a row comprising:
* resourceId references resource table), creatorId (references creator table), role (creator1, creator2 etc.) and order (1,2, 3 etc.
* for first author, second author, third author etc.)
* resourcecreatorCreatorMain is the creatorId that is the main creator for that resource used when ordering lists.
*/
	private function rewriteResourceCreator()
	{
		$this->db->query('CREATE TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'temprc` 
			(`resourcecreatorId` int(11) NOT NULL AUTO_INCREMENT,
			`resourcecreatorResourceId` int(11),
			`resourcecreatorCreatorId` int(11) DEFAULT NULL,
			`resourcecreatorOrder` int(11) DEFAULT NULL,
			`resourcecreatorRole` int(11) DEFAULT NULL,
			`resourcecreatorCreatorMain` int(11) DEFAULT NULL,
			`resourcecreatorCreatorSurname` varchar(255) DEFAULT NULL,
			PRIMARY KEY (`resourcecreatorId`),
			INDEX (`resourcecreatorCreatorSurname`), 
			INDEX (`resourcecreatorResourceId`), 
			INDEX (`resourcecreatorCreatorId`)) 
			ENGINE=MyISAM CHARSET=utf8
			');
		$creatorFields = array('resourcecreatorCreator1', 'resourcecreatorCreator2', 'resourcecreatorCreator3', 
			'resourcecreatorCreator4', 'resourcecreatorCreator5');
		$recordSet = $this->db->select('resource_creator', array('resourcecreatorId', 'resourcecreatorCreator1', 
			'resourcecreatorCreator2', 'resourcecreatorCreator3', 'resourcecreatorCreator4', 'resourcecreatorCreator5'));
		while($row = $this->db->fetchRow($recordSet))
		{
			$creatorMain = FALSE;
			foreach($creatorFields as $creatorField)
			{
				if(!$row[$creatorField])
					continue;
				$order = 1;
				$creatorIds = explode(',', $row[$creatorField]);
				foreach($creatorIds as $creatorId)
				{
					$fields = $values = array();
					$fields[] = 'resourcecreatorResourceId';
					$values[] = $row['resourcecreatorId'];
					$fields[] = 'resourcecreatorCreatorId';
					$values[] = $creatorId;
					$fields[] = 'resourcecreatorOrder';
					$values[] = $order;
					if($creatorField == 'resourcecreatorCreator1')
					{
						$fields[] = 'resourcecreatorRole';
						$values[] = 1;
						if(!$creatorMain)
						{
							$fields[] = 'resourcecreatorCreatorMain';
							$values[] = $creatorMain = $creatorId;
						}
						else
						{
							$fields[] = 'resourcecreatorCreatorMain';
							$values[] = $creatorMain;
						}
					}
					else if($creatorField == 'resourcecreatorCreator2')
					{
						$fields[] = 'resourcecreatorRole';
						$values[] = 2;
						if(!$creatorMain)
						{
							$fields[] = 'resourcecreatorCreatorMain';
							$values[] = $creatorMain = $creatorId;
						}
						else
						{
							$fields[] = 'resourcecreatorCreatorMain';
							$values[] = $creatorMain;
						}
					}
					else if($creatorField == 'resourcecreatorCreator3')
					{
						$fields[] = 'resourcecreatorRole';
						$values[] = 3;
						if(!$creatorMain)
						{
							$fields[] = 'resourcecreatorCreatorMain';
							$values[] = $creatorMain = $creatorId;
						}
						else
						{
							$fields[] = 'resourcecreatorCreatorMain';
							$values[] = $creatorMain;
						}
					}
					else if($creatorField == 'resourcecreatorCreator4')
					{
						$fields[] = 'resourcecreatorRole';
						$values[] = 4;
						if(!$creatorMain)
						{
							$fields[] = 'resourcecreatorCreatorMain';
							$values[] = $creatorMain = $creatorId;
						}
						else
						{
							$fields[] = 'resourcecreatorCreatorMain';
							$values[] = $creatorMain;
						}
					}
					else if($creatorField == 'resourcecreatorCreator5')
					{
						$fields[] = 'resourcecreatorRole';
						$values[] = 5;
						if(!$creatorMain)
						{
							$fields[] = 'resourcecreatorCreatorMain';
							$values[] = $creatorMain = $creatorId;
						}
						else
						{
							$fields[] = 'resourcecreatorCreatorMain';
							$values[] = $creatorMain;
						}
					}
					$this->db->formatConditions(array('creatorId' => $creatorMain));
					$recordSet2 = $this->db->select('creator', 'creatorSurname');
					while($surname = $this->db->fetchOne($recordSet2))
					{
						if($surname)
						{
							$fields[] = 'resourcecreatorCreatorSurname';
							$values[] = $surname;
						}
					}
					$this->db->insert('temprc', $fields, $values);
					++$order;
				}
			}
		}
		$this->db->freeSet($recordSet);
// Select, and insert into temprc, those resourceIds without a creator
		$this->db->formatConditions($this->db->formatFields('resourceId') . ' NOT IN ' . 
			$this->db->subQuery($this->db->selectNoExecute('resource_creator', 'resourcecreatorId'), FALSE, FALSE));
		$recordSet = $this->db->select('resource', 'resourceId');
		while($row = $this->db->fetchRow($recordSet))
		{
			$fields = $values = array();
			$fields[] = 'resourcecreatorResourceId';
			$values[] = $row['resourceId'];
			$this->db->insert('temprc', $fields, $values);
		}
		$this->db->freeSet($recordSet);
		$this->db->query('DROP TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource_creator`');
		$this->db->query('RENAME TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'temprc` TO `' . 
			$this->config->WIKINDX_DB_TABLEPREFIX . 'resource_creator`');
	}
/**
* Rewrite resource_keyword table
*/
	private function rewriteResourceKeyword()
	{
		$this->db->query('CREATE TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'temprk` 
			(`resourcekeywordId` int(11) NOT NULL AUTO_INCREMENT,
			`resourcekeywordResourceId` int(11) DEFAULT NULL,
			`resourcekeywordQuoteId` int(11) DEFAULT NULL,
			`resourcekeywordParaphraseId` int(11) DEFAULT NULL,
			`resourcekeywordMusingId` int(11) DEFAULT NULL,
			`resourcekeywordKeywordId` int(11) DEFAULT NULL,
			PRIMARY KEY (`resourcekeywordId`),
			INDEX (`resourcekeywordKeywordId`),
			INDEX (`resourcekeywordResourceId`)) 
			ENGINE=MyISAM CHARSET=utf8
			');
		$this->db->formatConditions($this->db->formatFields('resourcekeywordKeywords') . ' IS NOT NULL');
		$recordSet = $this->db->select('resource_keyword', array('resourcekeywordId', 'resourcekeywordKeywords'));
		while($row = $this->db->fetchRow($recordSet))
		{
			foreach(explode(',', $row['resourcekeywordKeywords']) as $kId)
				$this->db->insert('temprk', array('resourcekeywordResourceId', 'resourcekeywordKeywordId'), 
					array($row['resourcekeywordId'], $kId));
		}
		$this->db->freeSet($recordSet);
		$this->db->formatConditions($this->db->formatFields('resourcequoteKeywords') . ' IS NOT NULL');
		$recordSet = $this->db->select('resource_quote', array('resourcequoteId', 'resourcequoteKeywords'));
		while($row = $this->db->fetchRow($recordSet))
		{
			foreach(explode(',', $row['resourcequoteKeywords']) as $kId)
				$this->db->insert('temprk', array('resourcekeywordQuoteId', 'resourcekeywordKeywordId'), 
					array($row['resourcequoteId'], $kId));
		}
		$this->db->freeSet($recordSet);
		$this->db->formatConditions($this->db->formatFields('resourceparaphraseKeywords') . ' IS NOT NULL');
		$recordSet = $this->db->select('resource_paraphrase', array('resourceparaphraseId', 'resourceparaphraseKeywords'));
		while($row = $this->db->fetchRow($recordSet))
		{
			foreach(explode(',', $row['resourceparaphraseKeywords']) as $kId)
				$this->db->insert('temprk', array('resourcekeywordParaphraseId', 'resourcekeywordKeywordId'), 
					array($row['resourceparaphraseId'], $kId));
		}
		$this->db->freeSet($recordSet);
		$this->db->formatConditions($this->db->formatFields('resourcemusingKeywords') . ' IS NOT NULL');
		$recordSet = $this->db->select('resource_musing', array('resourcemusingId', 'resourcemusingKeywords'));
		while($row = $this->db->fetchRow($recordSet))
		{
			foreach(explode(',', $row['resourcemusingKeywords']) as $kId)
				$this->db->insert('temprk', array('resourcekeywordMusingId', 'resourcekeywordKeywordId'), 
					array($row['resourcemusingId'], $kId));
		}
		$this->db->freeSet($recordSet);
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource_quote` DROP COLUMN `resourcequoteKeywords`');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
			'resource_paraphrase` DROP COLUMN `resourceparaphraseKeywords`');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource_musing` DROP COLUMN `resourcemusingKeywords`');
		$this->db->query('DROP TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource_keyword`');
		$this->db->query('RENAME TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'temprk` TO `' . 
			$this->config->WIKINDX_DB_TABLEPREFIX . 'resource_keyword`');
	}
/**
* Rewrite resource_category table
*/
	private function rewriteResourceCategory()
	{
		$this->db->query('CREATE TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'temprc` 
			(`resourcecategoryId` int(11) NOT NULL AUTO_INCREMENT,
			`resourcecategoryResourceId` int(11) DEFAULT NULL,
			`resourcecategoryCategoryId` int(11) DEFAULT NULL,
			`resourcecategorySubcategoryId` int(11) DEFAULT NULL,
			PRIMARY KEY (`resourcecategoryId`),
			INDEX (`resourcecategoryCategoryId`),
			INDEX (`resourcecategoryResourceId`)) 
			ENGINE=MyISAM CHARSET=utf8
			');
		$recordSet = $this->db->select('resource_category', array('resourcecategoryId', 'resourcecategoryCategories'));
		while($row = $this->db->fetchRow($recordSet))
		{
			foreach(explode(',', $row['resourcecategoryCategories']) as $cId)
				$this->db->insert('temprc', array('resourcecategoryResourceId', 'resourcecategoryCategoryId'), 
					array($row['resourcecategoryId'], $cId));
		}
		$this->db->freeSet($recordSet);
		$this->db->query('DROP TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource_category`');
		$this->db->query('RENAME TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'temprc` TO `' . 
			$this->config->WIKINDX_DB_TABLEPREFIX . 'resource_category`');
	}
/**
* rewrite bibliographies
*/
	private function rewriteBibliographiesGroups()
	{
		$this->db->query('CREATE TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'user_bibliography_resource` 
			(`userbibliographyresourceId` int(11) NOT NULL AUTO_INCREMENT,
			`userbibliographyresourceBibliographyId` int(11) DEFAULT NULL,
			`userbibliographyresourceResourceId` int(11) DEFAULT NULL,
			PRIMARY KEY (`userbibliographyresourceId`),
			INDEX (`userbibliographyresourceResourceId`)) 
			ENGINE=MyISAM CHARSET=utf8
			');
		$this->db->query('CREATE TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'user_groups_users` 
			(`usergroupsusersId` int(11) NOT NULL AUTO_INCREMENT,
			`usergroupsusersGroupId` int(11) DEFAULT NULL,
			`usergroupsusersUserId` int(11) DEFAULT NULL,
			PRIMARY KEY (`usergroupsusersId`)) 
			ENGINE=MyISAM CHARSET=utf8
			');
		$recordSet = $this->db->select('user_bibliography', '*');
		while($row = $this->db->fetchRow($recordSet))
		{
			if($row['userbibliographyBibliography'])
			{
				foreach(explode(',', $row['userbibliographyBibliography']) as $rId)
					$this->db->insert('user_bibliography_resource', 
						array('userbibliographyresourceBibliographyId', 'userbibliographyresourceResourceId'), 
						array($row['userbibliographyId'], $rId));
			}
		}
		$this->db->freeSet($recordSet);
		$recordSet = $this->db->select('user_groups', array('usergroupsId', 'usergroupsUserIds', 'usergroupsBibliographyIds'));
		while($row = $this->db->fetchRow($recordSet))
		{
			foreach(explode(',', $row['usergroupsUserIds']) as $uId)
				$this->db->insert('user_groups_users', 
					array('usergroupsusersGroupId', 'usergroupsusersUserId'), 
					array($row['usergroupsId'], $uId));
		}
		$this->db->freeSet($recordSet);
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'user_bibliography` DROP COLUMN 
			`userbibliographyBibliography`');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'user_groups` DROP COLUMN 
			`usergroupsUserIds`');
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'user_groups` DROP COLUMN 
			`usergroupsBibliographyIds`');
	}
/**
* Create new tables user_tags and resource_user_tags
*/
	private function upgrade40CreateUsertags()
	{
		$this->db->query('CREATE TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'user_tags` 
			(`id` int(11) NOT NULL AUTO_INCREMENT,
			`userId` int(11),
			`tag` varchar(255),
			PRIMARY KEY (`id`)) 
			ENGINE=MyISAM CHARSET=utf8
			');
		$this->db->query('CREATE TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource_user_tags` 
			(`id` int(11) NOT NULL AUTO_INCREMENT,
			`tagId` int(11),
			`resourceId` int(11),
			PRIMARY KEY (`id`)) 
			ENGINE=MyISAM CHARSET=utf8
			');
	}
/**
* Create new table subcategory
*/
	private function upgrade40CreateSubcategory()
	{
		$this->db->query('CREATE TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'subcategory` 
			(`id` int(11) NOT NULL AUTO_INCREMENT,
			`categoryId` int(11),
			`subcategory` varchar(255),
			PRIMARY KEY (`id`)) 
			ENGINE=MyISAM CHARSET=utf8
			');
	}
/**
* Create new table resource_attachments and transfer attachments fields to it.
*
* Delete attachment rows if attachments don't exist.
*/ 
	private function upgrade40CreateResourceAttachments()
	{

// create statistics table and add resource view and attachment download data to it
		$this->db->query('CREATE TABLE IF NOT EXISTS `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'statistics` 
			(`id` int(11) AUTO_INCREMENT NOT NULL, 
			`resourceId` int(11) NOT NULL,
			`attachmentId` int(11) default NULL,
			`statistics` text, 
			PRIMARY KEY (`id`)) 
			ENGINE=MyISAM CHARSET=utf8
			');
		if(!$this->session->getVar('stage3UpgradeContinueView'))
			$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'statistics` ADD INDEX(`resourceId`)');
// First insert statistics rows by averaging out total resource views over months resource has been available.
// If we've paused execution, ensure we only select those that have not yet been done.
		$this->db->formatConditions(array('attachmentId' => ' IS NULL'));
		$subStmt = $this->db->subQuery($this->db->selectNoExecute('statistics', 'resourceId'), FALSE, FALSE, TRUE);
		$this->db->formatConditions($this->db->formatFields('resource_misc.id') . $this->db->inClause($subStmt, TRUE));
		$recordset = $this->db->query('SELECT `' . 
			$this->config->WIKINDX_DB_TABLEPREFIX . "resource_misc`.`id` AS `id`, 
			`accesses`, 
			`timestampAdd`, 
			date_format(`timestampAdd`, '%Y%m') AS `date`, 
			period_diff(date_format(now(), '%Y%m'), date_format(`timestampAdd`, '%Y%m')) AS `months` FROM `" . 
			$this->config->WIKINDX_DB_TABLEPREFIX . 'resource_misc` LEFT JOIN `' . 
			$this->config->WIKINDX_DB_TABLEPREFIX . 'resource_timestamp` ON `' . 
				$this->config->WIKINDX_DB_TABLEPREFIX . 'resource_timestamp`.`id` = `' . 
				$this->config->WIKINDX_DB_TABLEPREFIX . 'resource_misc`.`id`');
		$count = 0;
		while($row = $this->db->fetchRow($recordset))
		{
			$array = array();
			$year = substr($row['date'], 0, 4);
			$month = substr($row['date'], 4, 2);
			if(!$row['months'])
				$array[$row['date']] = $row['accesses'];
			else
			{
				$average = ceil($row['accesses'] / $row['months']);
				for($i = 0; $i < $row['months']; $i++)
				{
					$addYears = floor($i/12);
					$addMonths = $i % 12;
					$monthPart = $month + $addMonths;
					if($monthPart > 12)
					{
						++$addYears;
						$monthPart = $monthPart - 12;
					}
					$monthPart = $monthPart < 10 ? '0' . $monthPart : (string)$monthPart;
					$yearPart = $year + $addYears;
					$date = (string)$yearPart . $monthPart;
					$array[$date] = $average;
				}
			}
			$this->db->insert('statistics', array('resourceId', 'statistics'), 
				array($row['id'], base64_encode(serialize($array))));
			$this->db->formatConditions(array('id' => $row['id']));
			$this->db->update('resource_misc', array('accessesPeriod' => $average));
			$count++;
// Check we have more than 3 seconds buffer before max_execution_time times out.
			if((time() - $this->oldTime) >= (ini_get("max_execution_time") - 3))
			{
				$this->session->setVar('stage3UpgradeContinueView', TRUE);
				$this->checkStatus('stage4_1__3');
				$this->stageInterruptMessage = "stage3 continuing: $count view statistics created this pass.&nbsp;&nbsp;";
				$this->pauseExecution('stage4_1__3', 'stage4_1__3');
			}
		}
		$this->db->freeSet($recordset);
		if(!$this->session->getVar('stage3UpgradeContinueAttach')) // First time through and no continue execution
		{
			$this->db->query('RENAME TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'attachments` TO `' . 
				$this->config->WIKINDX_DB_TABLEPREFIX . 'resource_attachments`');
			$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
				'resource_attachments` CHANGE `filename` `fileName` VARCHAR(255) NOT NULL');
			$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
				'resource_attachments` ADD COLUMN `downloads` INT(11) DEFAULT 0');
			$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
				'resource_attachments` ADD COLUMN `downloadsPeriod` INT(11) DEFAULT 0');
			$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
				'resource_attachments` ADD COLUMN `primary` VARCHAR(1) DEFAULT \'N\'');
			$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
				'resource_attachments` ADD COLUMN `timestamp` DATETIME DEFAULT \'00/00/00 00:00:00\'');
			if(!isset($this->config->WIKINDX_ATTACHMENTS_DIR))
				$dirName = "attachments";
			else
				$dirName = $this->config->WIKINDX_ATTACHMENTS_DIR;
			$recordSet = $this->db->select('resource_attachments', array('id', 'hashFilename'));
			$deletes = array();
			while($row = $this->db->fetchRow($recordSet))
			{
				if(!file_exists("$dirName/" . $row['hashFilename']))
					$deletes[] = $row['id'];			
			}
			$this->db->freeSet($recordSet);
			if(!empty($deletes)) // discard these entries as there is no attachment
			{
				$this->db->formatConditionsOneField($deletes, 'id');
				$this->db->delete('resource_attachments');
			}
			$this->db->leftJoin('resource_timestamp', 'resource_timestamp.id', 'resource_misc.id');
			$recordset = $this->db->select('resource_misc', array(array('resource_misc.id' => 'id'), 'attachDownloads', 'timestampAdd',));
			while($row = $this->db->fetchRow($recordset))
			{
				$this->db->formatConditions(array('resourceId' => $row['id']));
				$this->db->update('resource_attachments', 
					array('downloads' => $row['attachDownloads'], 'timestamp' => $row['timestampAdd']));
			}
			$this->db->freeSet($recordset);
		}
// Insert statistics rows by averaging out total attachment downloads over months attachment has been available
// If we've paused execution, ensure we only select those that have not yet been done.
		$this->db->formatConditions(array('attachmentId' => ' IS NOT NULL'));
		$subStmt = $this->db->subQuery($this->db->selectNoExecute('statistics', 'resourceId'), FALSE, FALSE, TRUE);
		$this->db->formatConditions($this->db->formatFields('resourceId') . $this->db->inClause($subStmt, TRUE));
		$recordset = $this->db->query("SELECT `id`, 
			`resourceId`, 
			`downloads`, 
			`timestamp`, 
			date_format(`timestamp`, '%Y%m') AS `date`, 
			period_diff(date_format(now(), '%Y%m'), date_format(`timestamp`, '%Y%m')) AS `months` FROM `" . 
			$this->config->WIKINDX_DB_TABLEPREFIX . 'resource_attachments`');
		$count = 0;
		while($row = $this->db->fetchRow($recordset))
		{
			$array = array();
			$year = substr($row['date'], 0, 4);
			$month = substr($row['date'], 4, 2);
			if(!$row['months'])
				$array[$row['date']] = $row['downloads'];
			else
			{
				$average = ceil($row['downloads'] / $row['months']);
				for($i = 0; $i < $row['months']; $i++)
				{
					$addYears = floor($i/12);
					$addMonths = $i % 12;
					$monthPart = $month + $addMonths;
					if($monthPart > 12)
					{
						++$addYears;
						$monthPart = $monthPart - 12;
					}
					$monthPart = $monthPart < 10 ? '0' . $monthPart : (string)$monthPart;
					$yearPart = $year + $addYears;
					$date = (string)$yearPart . $monthPart;
					$array[$date] = $average;
				}
			}
			$this->db->insert('statistics', array('resourceId', 'attachmentId', 'statistics'), 
				array($row['resourceId'], $row['id'], base64_encode(serialize($array))));
			$this->db->formatConditions(array('id' => $row['id']));
			$this->db->update('resource_attachments', array('downloadsPeriod' => $average));
			$count++;
// Check we have more than 3 seconds buffer before max_execution_time times out.
			if((time() - $this->oldTime) >= (ini_get("max_execution_time") - 3))
			{
				$this->session->setVar('stage3UpgradeContinueAttach', TRUE);
				$this->checkStatus('stage4_1__3');
				$this->stageInterruptMessage = "stage3 continuing: $count attachment statistics created this pass.&nbsp;&nbsp;";
				$this->pauseExecution('stage4_1__3', 'stage4_1__3');
			}
		}
		$this->db->freeSet($recordset);
	}
/**
* Create new table resource_text and transfer resource_note and resource_abstract fields to it
*/
	private function upgrade40CreateResourceText()
	{
		$html = FACTORY_HTML::getInstance();
		$this->db->query('CREATE TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource_text` 
			(`id` int(11) NOT NULL, 
			`note` text, 
			`abstract` text, 
			`urls` text,
			`urlText` text,
			`editUserIdNote` int(11) default NULL,
			`addUserIdNote` int(11) default NULL,
			`editUserIdAbstract` int(11) default NULL,
			`addUserIdAbstract` int(11) default NULL,
			PRIMARY KEY (`id`)) 
			ENGINE=MyISAM CHARSET=utf8
			');
		$ids = array();
// Transfer URLs (now stores multiple URLs so base_64 encode and serialize before storing)
		$recordSet = $this->db->select('resource', array('id', 'url'));
		while($row = $this->db->fetchRow($recordSet))
		{
			if($row['url'])
			{
				$fields = array('id', 'urls');
				$values = array($row['id'], base64_encode(serialize(array($row['url']))));
				$this->db->insert('resource_text', $fields, $values);
				$ids[$row['id']] = FALSE;
			}
		}
		$this->db->freeSet($recordSet);
// Transfer notes
		$recordSet = $this->db->select('resource_note', array('id', 'text', 'editUserIdNote', 'addUserIdNote'));
		while($row = $this->db->fetchRow($recordSet))
		{
			if($row['text'] && array_key_exists($row['id'], $ids)) // update row
			{
				$updateArray = array('note' => $this->utf8->smartUtf8_decode($html->nlToHtml($row['text'], TRUE)));
				if($row['editUserIdNote'])
					$updateArray['editUserIdNote'] = $row['editUserIdNote'];
				if($row['addUserIdNote'])
					$updateArray['addUserIdNote'] = $row['addUserIdNote'];
				$this->db->formatConditions(array('id' => $row['id']));
				$this->db->update('resource_text', $updateArray);
			}
			else if($row['text']) // insert
			{
				$fields = array('id', 'note');
				$values = array($row['id'], $this->utf8->smartUtf8_decode($html->nlToHtml($row['text'], TRUE)));
				if($row['editUserIdNote'])
				{
					$fields[] = 'editUserIdNote';
					$values[] = $row['editUserIdNote'];
				}
				if($row['addUserIdNote'])
				{
					$fields[] = 'addUserIdNote';
					$values[] = $row['addUserIdNote'];
				}
				$this->db->insert('resource_text', $fields, $values);
				$ids[$row['id']] = FALSE;
			}
		}
		$this->db->freeSet($recordSet);
// Transfer abstracts
		$recordSet = $this->db->select('resource_abstract', array('id', 'abstract', 'editUserIdAbstract', 'addUserIdAbstract'));
		while($row = $this->db->fetchRow($recordSet))
		{
			if($row['abstract'] && array_key_exists($row['id'], $ids)) // update row
			{
				$updateArray = array('abstract' => $this->utf8->smartUtf8_decode($html->nlToHtml($row['abstract'], TRUE)));
				if($row['editUserIdAbstract'])
					$updateArray['editUserIdAbstract'] = $row['editUserIdAbstract'];
				if($row['addUserIdAbstract'])
					$updateArray['addUserIdAbstract'] = $row['addUserIdAbstract'];
				$this->db->formatConditions(array('id' => $row['id']));
				$this->db->update('resource_text', $updateArray);
			}
			else if($row['abstract']) // insert
			{
				$fields = array('id', 'abstract');
				$values = array($row['id'], $this->utf8->smartUtf8_decode($html->nlToHtml($row['abstract'], TRUE)));
				if($row['editUserIdAbstract'])
				{
					$fields[] = 'editUserIdAbstract';
					$values[] = $row['editUserIdAbstract'];
				}
				if($row['addUserIdAbstract'])
				{
					$fields[] = 'addUserIdAbstract';
					$values[] = $row['addUserIdAbstract'];
				}
				$this->db->insert('resource_text', $fields, $values);
			}
		}
		$this->db->freeSet($recordSet);
		$this->db->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource` DROP COLUMN `url`');
// Drop abstract and note tables
		$this->db->query('DROP TABLE IF EXISTS `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource_note`');
		$this->db->query('DROP TABLE IF EXISTS `' . $this->config->WIKINDX_DB_TABLEPREFIX . 'resource_abstract`');
	}
/**
* v3.3 corrected additional backslashes that had crept in but forgot to deal with the resource_note table
*/
	private function upgrade40CorrectBackSlash()
	{
		$this->db->escapeString = TRUE;
		$resultset = $this->db->select('resource_note', array('id', 'text'));
		while($row = $this->db->fetchRow($resultset))
		{
			$update = array();
			$update['text'] = stripslashes(stripslashes($row['text']));
			$this->db->formatConditions(array('id' => $row['id']));
			$this->db->update('resource_note', $update);
		}
		$this->db->freeSet($resultset);
		$this->db->escapeString = FALSE;
		unset($update);
	}
/**
* If required to pause execution, store current position and any $tableArray arrays in session and present continution form to user
*
* @param string $function
* @param string $finished
* @param string $table Default is FALSE
*/
	private function pauseExecution($function, $finished, $table = FALSE)
	{
// Store data
		$form = FACTORY_FORM::getInstance();
		$this->session->setVar('upgrade_function', $function);
		if($table)
			$this->session->setVar('upgrade_table', $table);
		foreach($this->tables as $table)
		{
			$table .= 'Array';
			if(isset($this->{$table}))
				$tableArrays[$table] = $this->{$table};
		}
		if(isset($tableArrays))
			$this->session->setVar('upgrade_tableArrays', base64_encode(serialize($tableArrays)));
// Print form and die
//		$pString = "php.ini's max_execution time (" . ini_get("max_execution_time") . " seconds) was about 
//			to be exceeded.  Please click on the button to continue the upgrade.&nbsp;&nbsp;Do <b>not</b> click 
//			until each script has finished.";
		$pString = $this->html->p($this->html->b($this->messages->text("heading", "upgradeDB")));
		$pString .= $this->html->p($this->statusString);
		if($this->stageInterruptMessage)
			$pString .= $this->html->p($this->stageInterruptMessage);
		else
			$pString .= $this->html->p("`$finished` of " . $this->numStages . 
			" finished.  Please click on the button to continue the upgrade.&nbsp;&nbsp;Do <b>not</b> click 
			until each script has finished.");
		$pString .= $form->formHeader('continueExecution');
		if($function == 'stage4_1__15')
		{
			$pString .= $this->html->p($this->messages->text("config", "upgradeFixUTF8-1"));
			$pString .= $this->html->p($this->messages->text("config", "upgradeFixUTF8-2") . '&nbsp;&nbsp;' . $form->checkBox(FALSE, 'fixUTF8'));
		}
		$pString .= $this->html->p($form->formSubmit("Continue") . $form->formEnd());
		$this->close($pString);
	}
/**
* Continue execution of upgrade after a pause
*/
	private function continueExecution()
	{
		$function = $this->session->getVar('upgrade_function');
		$tableArrays = $this->session->getVar('upgrade_tableArrays');
		if($tableArrays)
		{
			$tableArrays = unserialize(base64_decode($tableArrays));
			foreach($tableArrays as $table => $array)
				$this->{$table} = $array;
		}
		$table = $this->session->getVar('upgrade_table');
		$this->session->clearArray('upgrade');
		if($function == 'upgrade40charToBin')
		{
			if($table)
				$this->upgrade40charToBin($table);
			else
				$this->upgrade40charToBin();
			$this->upgrade40Nulls();
			$this->db->query("ALTER DATABASE `" . $this->config->WIKINDX_DB . 
				"` DEFAULT CHARACTER SET utf8 COLLATE utf8_unicode_ci");
			$this->upgrade40Tables();
		}
		else if($function == 'upgrade40Tables')
		{
			if($table)
				$this->upgrade40Tables($table);
			else
				$this->upgrade40Tables();
		}
	}
/**
* pre v4.0 versions have a bug whereby creators may be duplicated in the database -- find these and fix.
*/
	private function upgrade40RemoveDupCreators()
	{
		$resultset = $this->db->query("SELECT `a`.`id` AS `aId`, `b`.`id` AS `bId`
			FROM `" . $this->config->WIKINDX_DB_TABLEPREFIX . "creator` AS a
			INNER JOIN `" . $this->config->WIKINDX_DB_TABLEPREFIX . "creator` AS b 
			ON 
			(`a`.`firstname` = `b`.`firstname` OR (`a`.`firstname` IS NULL AND `b`.`firstname` IS NULL))
			AND 
			(`a`.`initials` = `b`.`initials`  OR (`a`.`initials` IS NULL AND `b`.`initials` IS NULL))
			AND 
			(`a`.`prefix` = `b`.`prefix`  OR (`a`.`prefix` IS NULL AND `b`.`prefix` IS NULL))
			AND 
			`a`.`surname` = `b`.`surname` 
			AND 
			`a`.`id` != `b`.`id`
			");
		$dupCreators = $delCreators = array();
		$count = 0;
		while($row = $this->db->fetchRow($resultset))
		{
			if(array_search($row['aId'], $delCreators) !== FALSE)
				continue;
			if(array_search($row['aId'], $dupCreators) === FALSE)
			{
				$dupCreators[$row['aId']] = $row['bId'];
				$resultset2 = $this->db->query("SELECT `id`, `creator1`, `creator2`, `creator3`, `creator4`, `creator5`
					FROM `" . $this->config->WIKINDX_DB_TABLEPREFIX . "resource_creator`
					WHERE
					FIND_IN_SET('" . $row['bId'] . "', `creator1`) 
					OR 
					FIND_IN_SET('" . $row['bId'] . "', `creator2`) 
					OR 
					FIND_IN_SET('" . $row['bId'] . "', `creator3`) 
					OR 
					FIND_IN_SET('" . $row['bId'] . "', `creator4`) 
					OR 
					FIND_IN_SET('" . $row['bId'] . "', `creator5`)"
					);
				while($row2 = $this->db->fetchRow($resultset2))
				{		
					$updateArray = array();
					foreach(array('creator1', 'creator2', 'creator3', 'creator4', 'creator5') as $creator)
					{
// remove bId creator id and replace with aId if aId not already in set
						if($row2[$creator])
						{
							$oldArray = explode(",", $row2[$creator]);
// If $row['aId'] already in db field, simply remove $row['bId']
							if((array_search($row['aId'], $oldArray) !== FALSE) && 
								(($editIndex = array_search($row['bId'], $oldArray)) !== FALSE))
							{
								unset($oldArray[$editIndex]);
								$updateArray[$creator] = join(",", $oldArray);
							}
// else, if this creator field has $row['bId'] in it, replace it with $row['aId']
							else if(($editIndex = array_search($row['bId'], $oldArray)) !== FALSE)
							{
								$oldArray[$editIndex] = $row['aId'];
								$updateArray[$creator] = join(",", $oldArray);
							}
						}
					}
					if(!empty($updateArray))
					{
						$this->db->formatConditions(array('id' => $row2['id']));
						$this->db->update('resource_creator', $updateArray);
					}
				}
				if(array_search($row['bId'], $delCreators) === FALSE)
				{
					++$count;
					$delCreators[] = $row['bId'];
// delete bId
					$this->db->formatConditions(array('id' => $row['bId']));
					$this->db->delete('creator');
				}
			}
// Check we have more than 3 seconds buffer before max_execution_time times out.
			if((time() - $this->oldTime) >= (ini_get("max_execution_time") - 3))
			{
				$this->checkStatus('stage4_1__4');
				$this->stageInterruptMessage = "stage4 continuing: $count duplicate creators corrected this pass.&nbsp;&nbsp;";
				$this->pauseExecution('stage4_1__4', 'stage4_1__4');
			}
		}
		$this->db->freeSet($resultset);
	}
/**
* Remove duplicate collections
*/
	private function upgrade40RemoveDupCollections()
	{
		$resultset = $this->db->query("SELECT `a`.`id` AS `aId`, `b`.`id` AS `bId`
			FROM `" . $this->config->WIKINDX_DB_TABLEPREFIX . "collection` AS a
			INNER JOIN `" . $this->config->WIKINDX_DB_TABLEPREFIX . "collection` AS b 
			ON 
			(`a`.`collectionTitle` = `b`.`collectionTitle`)
			AND 
			(`a`.`collectionTitleShort` = `b`.`collectionTitleShort`  
				OR (`a`.`collectionTitleShort` IS NULL AND `b`.`collectionTitleShort` IS NULL))
			AND 
			(`a`.`collectionType` = `b`.`collectionType`)
			AND 
			`a`.`id` != `b`.`id`
			");
		$dupCollections = $delCollections = array();
		$count = 0;
		while($row = $this->db->fetchRow($resultset))
		{
			if(array_search($row['aId'], $dupCollections) === FALSE)
			{
				$dupCollections[$row['aId']] = $row['bId'];
				$resultset2 = $this->db->query("SELECT `id`, `collection`
					FROM `" . $this->config->WIKINDX_DB_TABLEPREFIX . "resource_misc`
					WHERE `collection` = '" . $row['bId'] . "'");
				while($row2 = $this->db->fetchRow($resultset2))
				{		
					$updateArray = array();
// remove bId collection id and replace with aId
					if($row2['collection'] && ($row2['collection'] == $row['bId']))
						$updateArray['collection'] = $row['aId'];
					if(!empty($updateArray))
					{
						$this->db->formatConditions(array('id' => $row2['id']));
						$this->db->update('resource_misc', $updateArray);
					}
				}
				if(array_search($row['bId'], $delCollections) === FALSE)
				{
					++$count;
					$delCollections[] = $row['bId'];
// delete bId
					$this->db->formatConditions(array('id' => $row['bId']));
					$this->db->delete('collection');
				}
			}
// Check we have more than 2 seconds buffer before max_execution_time times out.
			if((time() - $this->oldTime) >= (ini_get("max_execution_time") - 2))
			{
				$this->checkStatus('stage4_1__5');
				$this->stageInterruptMessage = "stage5 continuing: $count duplicate collections corrected this pass.&nbsp;&nbsp;";
				$this->pauseExecution('stage4_1__5', 'stage4_1__5');
			}
		}
		$this->db->freeSet($resultset);
	}
/**
* Store function in MySQL to check for invalid UTF8 code in latin1 database
*
* NOT USED
*/
	private function upgrade40StoreProcedure()
	{
		$this->db->query('DROP FUNCTION IF EXISTS is_invalid_utf8');
		$this->db->query('CREATE FUNCTION is_invalid_utf8(s BLOB) 
		RETURNS int 
		DETERMINISTIC 
			  BEGIN 
			    DECLARE i INT DEFAULT 1; 
			    DECLARE c INT; 
			    DECLARE num_expected_highs INT DEFAULT 0; 
			    WHILE (i <= length(s)) DO 
			      SET c = ord(mid(s, i, 1)); 
			      IF num_expected_highs > 0 THEN 
			        IF (c & 192)<>128 THEN  
			          RETURN 1; 
			        END IF; 
			        SET num_expected_highs = num_expected_highs - 1; 
			      ELSE 
			        IF (c & 224) = 192 THEN  
			          SET num_expected_highs = 1; 
			        ELSEIF (c & 240) = 224 THEN  
			          SET num_expected_highs = 2; 
			        ELSEIF (c & 248) = 240 THEN  
			          SET num_expected_highs = 3; 
			        ELSEIF (c & 128 = 128) THEN 
			          RETURN 1; 
			        END IF; 
			      END IF; 
			      SET i = (i+1); 
			    END WHILE; 
			    RETURN num_expected_highs; 
			  END');
	}
/**
* Rename fields in tables for v4.0 upgrade
*/
	private function upgrade40Rename()
	{
		$extra = ' NOT NULL AUTO_INCREMENT';
		$noAutoIncrement = array('resource_quote_text', 'resource_paraphrase_text', 'import_raw', 
			'resource_category', 'resource_creator', 'resource_keyword', 'resource_text', 'resource_page', 'resource_summary', 
			'resource_timestamp', 'resource_year');
		foreach($this->tables as $value)
		{
			$basicTable = $value;
			$table = '`' . $this->config->WIKINDX_DB_TABLEPREFIX . "$basicTable`";
			$recordset = $this->db->query("SHOW COLUMNS FROM $table");
			$change = array();
			if(($value == 'collection'))
				$change[] = ' CHANGE id collectionId int(11) NOT NULL AUTO_INCREMENT';
			else if(($value == 'publisher'))
				$change[] = ' CHANGE id publisherId int(11) NOT NULL AUTO_INCREMENT';
			else if($value == 'custom')
			{
				$change[] = ' CHANGE id customId int(11) NOT NULL AUTO_INCREMENT';
				$change[] = ' CHANGE custom_label ' . 'customLabel varchar(255)';
				$change[] = ' CHANGE custom_size ' . 'customSize varchar(255)';
			}
			else if($value == 'resource_custom')
			{
				while($row = $this->db->fetchRow($recordset))
				{
					$extra2 = '';
					if($row['Null'] == 'NO')
						$extra2 = ' NOT NULL';
					if($row['Default'] !== '')
						$extra2 .= ' DEFAULT \'' . $row['Default'] . '\'';
					if($row['Field'] == 'custom_long')
						$change[] = ' CHANGE ' . $row['Field'] . ' resourcecustomLong ' . $row['Type'] . $extra2;
					else if($row['Field'] == 'custom_short')
						$change[] = ' CHANGE ' . $row['Field'] . ' resourcecustomShort ' . $row['Type'] . $extra2;
					else
					{
						if($row['Field'] == 'id')
							$tempExtra = $extra;
						else
							$tempExtra = '';
						$change[] = ' CHANGE ' . $row['Field'] . ' resourcecustom'
							. ucfirst($row['Field']) . ' ' . stripslashes($row['Type']) . $tempExtra . $extra2;
					}
				}
			}
			else if($value == 'resource_misc')
			{
				while($row = $this->db->fetchRow($recordset))
				{
					$extra2 = '';
					if($row['Null'] == 'NO')
						$extra2 = ' NOT NULL';
					if($row['Default'] !== '')
						$extra2 .= ' DEFAULT \'' . $row['Default'] . '\'';
					if(($row['Field'] == 'miscField1') OR ($row['Field'] == 'miscField2') OR 
					($row['Field'] == 'miscField3') OR ($row['Field'] == 'miscField4') OR 
					($row['Field'] == 'miscField5') OR ($row['Field'] == 'miscField6'))
						$change[] = ' CHANGE ' . $row['Field'] . ' resourcemisc' . 
						str_replace('misc', '', $row['Field']) . ' ' . $row['Type'] . $extra2;
					else
					{
						if($row['Field'] == 'id')
							$tempExtra = $extra;
						else
							$tempExtra = '';
						$change[] = ' CHANGE ' . $row['Field'] . ' resourcemisc'
							. ucfirst($row['Field']) . ' ' . stripslashes($row['Type']) . $extra2;
					}
				}
			}
			else if($value == 'resource_musing')
			{
				while($row = $this->db->fetchRow($recordset))
				{
					$extra2 = '';
					if($row['Null'] == 'NO')
						$extra2 = ' NOT NULL';
					if($row['Default'] !== '')
						$extra2 .= ' DEFAULT \'' . $row['Default'] . '\'';
					if($row['Field'] == 'page_start')
						$change[] = ' CHANGE ' . $row['Field'] . ' resourcemusingPageStart ' . $row['Type'] . $extra2;
					else if($row['Field'] == 'page_end')
						$change[] = ' CHANGE ' . $row['Field'] . ' resourcemusingPageEnd ' . $row['Type'] . $extra2;
					else if($row['Field'] == 'musing_keywords')
						$change[] = ' CHANGE ' . $row['Field'] . ' resourcemusingKeywords ' . $row['Type'] . $extra2;
					else
					{
						if($row['Field'] == 'id')
							$tempExtra = $extra;
						else
							$tempExtra = '';
						$change[] = ' CHANGE ' . $row['Field'] . ' resourcemusing'
							. ucfirst($row['Field']) . ' ' . stripslashes($row['Type']) . $tempExtra . $extra2;
					}
				}
			}
			else if($value == 'resource_musing_text')
			{
				while($row = $this->db->fetchRow($recordset))
				{
					$extra2 = '';
					if($row['Null'] == 'NO')
						$extra2 = ' NOT NULL';
					if($row['Default'] !== '')
						$extra2 .= ' DEFAULT \'' . $row['Default'] . '\'';
					if($row['Field'] == 'musingPrivate')
						$change[] = ' CHANGE ' . $row['Field'] . ' resourcemusingtextPrivate ' . $row['Type'] . $extra2;
					else
					{
						if($row['Field'] == 'id')
							$tempExtra = $extra;
						else
							$tempExtra = '';
						$change[] = ' CHANGE ' . $row['Field'] . ' resourcemusingtext'
							. ucfirst($row['Field']) . ' ' . stripslashes($row['Type']) . $extra2;
					}
				}
			}
			else if($value == 'resource_paraphrase')
			{
				while($row = $this->db->fetchRow($recordset))
				{
					$extra2 = '';
					if($row['Null'] == 'NO')
						$extra2 = ' NOT NULL';
					if($row['Default'] !== '')
						$extra2 .= ' DEFAULT \'' . $row['Default'] . '\'';
					if($row['Field'] == 'page_start')
						$change[] = ' CHANGE ' . $row['Field'] . ' resourceparaphrasePageStart ' . $row['Type'] . $extra2;
					else if($row['Field'] == 'page_end')
						$change[] = ' CHANGE ' . $row['Field'] . ' resourceparaphrasePageEnd ' . $row['Type'] . $extra2;
					else if($row['Field'] == 'paraphrase_keywords')
						$change[] = ' CHANGE ' . $row['Field'] . ' resourceparaphraseKeywords ' . $row['Type'] . $extra2;
					else
					{
						if($row['Field'] == 'id')
							$tempExtra = $extra;
						else
							$tempExtra = '';
						$change[] = ' CHANGE ' . $row['Field'] . ' resourceparaphrase'
							. ucfirst($row['Field']) . ' ' . stripslashes($row['Type']) . $tempExtra . $extra2;
					}
				}
			}
			else if($value == 'resource_paraphrase_comment')
			{
				while($row = $this->db->fetchRow($recordset))
				{
					$extra2 = '';
					if($row['Null'] == 'NO')
						$extra2 = ' NOT NULL';
					if($row['Default'] !== '')
						$extra2 .= ' DEFAULT \'' . $row['Default'] . '\'';
					if($row['Field'] == 'paraphrasePrivate')
						$change[] = ' CHANGE ' . $row['Field'] . ' resourceparaphrasecommentPrivate ' . $row['Type'] . $extra2;
					else
					{
						if($row['Field'] == 'id')
							$tempExtra = $extra;
						else
							$tempExtra = '';
						$change[] = ' CHANGE ' . $row['Field'] . ' resourceparaphrasecomment'
							. ucfirst($row['Field']) . ' ' . stripslashes($row['Type']) . $tempExtra . $extra2;
					}
				}
			}
			else if($value == 'resource_quote')
			{
				while($row = $this->db->fetchRow($recordset))
				{
					$extra2 = '';
					if($row['Null'] == 'NO')
						$extra2 = ' NOT NULL';
					if($row['Default'] !== '')
						$extra2 .= ' DEFAULT \'' . $row['Default'] . '\'';
					if($row['Field'] == 'page_start')
						$change[] = ' CHANGE ' . $row['Field'] . ' resourcequotePageStart ' . $row['Type'] . $extra2;
					else if($row['Field'] == 'page_end')
						$change[] = ' CHANGE ' . $row['Field'] . ' resourcequotePageEnd ' . $row['Type'] . $extra2;
					else if($row['Field'] == 'quote_keywords')
						$change[] = ' CHANGE ' . $row['Field'] . ' resourcequoteKeywords ' . $row['Type'] . $extra2;
					else
					{
						if($row['Field'] == 'id')
							$tempExtra = $extra;
						else
							$tempExtra = '';
						$change[] = ' CHANGE ' . $row['Field'] . ' resourcequote'
							. ucfirst($row['Field']) . ' ' . stripslashes($row['Type']) . $tempExtra . $extra2;
					}
				}
			}
			else if($value == 'resource_quote_comment')
			{
				while($row = $this->db->fetchRow($recordset))
				{
					$extra2 = '';
					if($row['Null'] == 'NO')
						$extra2 = ' NOT NULL';
					if($row['Default'] !== '')
						$extra2 .= ' DEFAULT \'' . $row['Default'] . '\'';
					if($row['Field'] == 'quotePrivate')
						$change[] = ' CHANGE ' . $row['Field'] . ' resourcequotecommentPrivate ' . $row['Type'] . $extra2;
					else
					{
						if($row['Field'] == 'id')
							$tempExtra = $extra;
						else
							$tempExtra = '';
						$change[] = ' CHANGE ' . $row['Field'] . ' resourcequotecomment'
							. ucfirst($row['Field']) . ' ' . stripslashes($row['Type']) . $tempExtra . $extra2;
					}
				}
			}
			else
			{
				while($row = $this->db->fetchRow($recordset))
				{
					$extra2 = '';
					if($row['Null'] == 'NO')
						$extra2 = ' NOT NULL';
					if($row['Default'] !== '')
						$extra2 .= ' DEFAULT \'' . $row['Default'] . '\'';
					if(($row['Field'] == 'id') && (array_search($value, $noAutoIncrement) === FALSE))
						$tempExtra = $extra;
					else
						$tempExtra = '';
					$change[] = ' CHANGE `' . $row['Field'] . '` ' . str_replace('_', '', $basicTable)
						. ucfirst($row['Field']) . ' ' . stripslashes($row['Type']) . $tempExtra . $extra2;
				}
			}
			$this->db->query('ALTER TABLE ' . $table . join(',', $change));
			$this->db->freeSet($recordset);
		}
	}
/**
* Update VARCHARS to account for increased size of UTF8 fields (3* space required) and then temp and change char types to binary types on some tables
* @param string $table Default is FALSE
*/
	private function upgrade40charToBin($table = FALSE)
	{
//		if($table) // continuing after pause
//			$tables = array_slice($tables, array_search($table, $tables) + 1);
		foreach($this->tables as $value)
		{
			$change = $varchar = array();
			$basicTable = $value;
			$table = '`' . $this->config->WIKINDX_DB_TABLEPREFIX . "$basicTable`";
			$fields = $conditions = array();
			$recordset = $this->db->query("SHOW COLUMNS FROM $table");
			while($row = $this->db->fetchRow($recordset))
			{
				if(($row['Type'] == 'varchar(10)') || $row['Type'] == 'varchar(30)')
				{
					$fields[] = $row['Field'];
					$varchar[] = '`' . $row['Field'] . '` `'  . $row['Field'] . '` varchar(255)';
				}
				else if($row['Type'] == 'varchar(255)')
					$fields[] = $row['Field'];
				else if($row['Type'] == 'mediumtext')
				{
					$fields[] = $row['Field'];
					$varchar[] = '`' . $row['Field'] . '` `'  . $row['Field'] . '` text';
				}
				else if($row['Type'] == 'text')
					$fields[] = $row['Field'];
			}
			$this->db->freeSet($recordset);
			if(!empty($fields))
				$this->upgrade40CheckForValidUtf8($basicTable, $fields);
// reset database conditions, join etc (upgrade40CheckForValidUtf8) may not have executed SQL query but condition still set
			$this->db->resetSubs();
			if(!empty($varchar))
			{
				foreach($varchar as $new)
					$change[] = ' CHANGE ' . $new;
				$this->db->query('ALTER TABLE ' . $table . join(',', $change));
			}
		}
		$tables = array('resource_attachments', 'category', 'collection', 'config', 'creator', 
			'custom', 'keyword', 'news', 'papers', 'publisher', 'resource', 
			'resource_custom', 'resource_musing', 'resource_musing_text', 'resource_text', 'resource_page', 
			'resource_paraphrase', 'resource_paraphrase_comment', 'resource_paraphrase_text', 
			'resource_quote', 'resource_quote_comment', 'resource_quote_text', 'resource_year', 'tag', 
			'users', 'user_bibliography', 'user_groups', 'user_register', 
			'user_tags', 'resource_user_tags', 'subcategory', 'statistics');
		foreach($tables as $value)
		{
			$change = $varchar = array();
			$basicTable = $value;
			$table = '`' . $this->config->WIKINDX_DB_TABLEPREFIX . "$basicTable`";
			$recordset = $this->db->query("SHOW COLUMNS FROM $table");
			while($row = $this->db->fetchRow($recordset))
			{
				if(($row['Type'] == 'varchar(255)'))
					$varchar[] = '`' . $row['Field'] . '` `'  . $row['Field'] . '` varbinary(255)';
				else if($row['Type'] == 'text')
					$varchar[] = '`' . $row['Field'] . '` `'  . $row['Field'] . '` blob';
			}
			$this->db->freeSet($recordset);
			if(!empty($varchar))
			{
				foreach($varchar as $new)
					$change[] = ' CHANGE ' . $new;
			}
			$this->db->query('ALTER TABLE ' . $table . join(',', $change));
		}
	}
/**
* Check for valid UTF-8.
*
* If invalid code found, store value in array for updating to database after UTF8 upgrade
* @param string $basicTable
* @param array $fields
*/
	private function upgrade40CheckForValidUtf8($basicTable, $fields)
	{
		if(($basicTable == 'cache') || ($basicTable == 'database_summary') ||
			($basicTable == 'resource_keyword') || ($basicTable == 'resource_category') || 
			($basicTable == 'resource_creator'))
			return;
		if($basicTable != 'config') // 'config' table has no id field
		{
			$id = str_replace('_', '', $basicTable) . 'Id';
			$fields[] = $id;
		}
		else
			$id = 'null';
		$resultset = $this->db->select($basicTable, $fields);
		$tableArray = $basicTable . 'Array';
		while($row = $this->db->fetchRow($resultset))
		{
			foreach($fields as $field)
			{
				if(($basicTable != 'config') && ($field == $id))
					continue;
				if(function_exists('mb_detect_encoding'))
				{
					$encoding = mb_detect_encoding($row[$field]);
					if(($encoding != 'ASCII') && ($encoding != 'UTF-8'))
					{
						if(!$encoding) // no idea what this is, so store for re-writing after upgrade later
							$value = $row[$field];
						else
						{
							if(!$value = iconv($encoding, 'utf-8', $row[$field]))
								$value = $row[$field];
						}
						if($basicTable == 'config')
							$this->{$tableArray}[][$field] = $value;
						else
							$this->{$tableArray}[$row[$id]][$field] = $value;
					}
				}
			}
		}
		$this->db->freeSet($resultset);
	}
/**
* Check for valid UTF-8.
*
* If invalid code found, store value in array for updating to database after UTF8 upgrade
* NOT USED
* @param string $basicTable
* @param array $fields
*/
	private function upgrade40CheckForValidUtf8Old($basicTable, $fields)
	{
		if(($basicTable == 'cache') || ($basicTable == 'database_summary') ||
			($basicTable == 'resource_keyword') || ($basicTable == 'resource_category') || 
			($basicTable == 'resource_creator'))
			return;
		if($basicTable != 'config')
		{
			$id = str_replace('_', '', $basicTable) . 'Id';
			$fields[] = $id;
		}
		$resultset = $this->db->select($basicTable, $fields);
		$tableArray = $basicTable . 'Array';
		while($row = $this->db->fetchRow($resultset))
			$this->{$tableArray}[] = $row;
		$this->db->freeSet($resultset);
	}
/**
* Remove user's sessions and caches
*/
	private function upgrade40Nulls()
	{
// Remove any existing user session
		$this->db->updateNull('users', 'userSession');
// Remove cache fields so that v4's UTF8 handling and ordering of creators, keywords etc. is handled properly
		$this->db->updateNull('cache', 'resourceCreators');
		$this->db->updateNull('cache', 'metadataCreators');
		$this->db->updateNull('cache', 'resourceKeywords');
		$this->db->updateNull('cache', 'metadataKeywords');
		$this->db->updateNull('cache', 'quoteKeywords');
		$this->db->updateNull('cache', 'paraphraseKeywords');
		$this->db->updateNull('cache', 'musingKeywords');
		$this->db->updateNull('cache', 'resourcePublishers');
		$this->db->updateNull('cache', 'metadataPublishers');
		$this->db->updateNull('cache', 'conferenceOrganisers');
		$this->db->updateNull('cache', 'resourceCollections');
		$this->db->updateNull('cache', 'metadataCollections');
		$this->db->updateNull('cache', 'resourceCollectionTitles');
		$this->db->updateNull('cache', 'resourceCollectionShorts');
	}
/**
* recreate the 4.0 cache
*/
	private function recreate40Cache()
	{
		$keyword = FACTORY_KEYWORD::getInstance();
		$creator = FACTORY_CREATOR::getInstance();
		$publisher = FACTORY_PUBLISHER::getInstance();
		$collection = FACTORY_COLLECTION::getInstance();
		$creator->grabAll();
		$creator->grabAll(FALSE, FALSE, TRUE);  // creators in resources with metadata
		$keyword->grabAll(); // default resource_keyword table
		$keyword->grabAll(FALSE, 'resource');
		$keyword->grabAll(FALSE, 'quote');
		$keyword->grabAll(FALSE, 'paraphrase');
		$keyword->grabAll(FALSE, 'musing');
		$publisher->grabAll();
		$publisher->grabAll(FALSE, FALSE, FALSE, TRUE); // publishers in resources with metadata
		$collection->grabAll();
		$collection->grabAll(FALSE, FALSE, FALSE, TRUE); // collections in resources with metadata
	}
/**
* Upgrade tables to UTF-8 then reverse field type change for some tables from binary back to var
*
* @param string $tableContinue Default is FALSE
*/
	private function upgrade40Tables($tableContinue = FALSE)
	{
		foreach($this->tables as $value)
		{
			$basicTable = $value;
			$table = '`' . $this->config->WIKINDX_DB_TABLEPREFIX . "$basicTable`";
			$this->db->query('ALTER TABLE ' . $table . ' CONVERT TO CHARACTER SET utf8 COLLATE utf8_unicode_ci');
		}
		$tables = array('resource_attachments', 'category', 'collection', 'config', 'creator', 
			'custom', 'keyword', 'news', 'papers', 'publisher', 'resource', 
			'resource_custom', 'resource_musing', 'resource_musing_text', 'resource_text', 'resource_page', 
			'resource_paraphrase', 'resource_paraphrase_comment', 'resource_paraphrase_text', 
			'resource_quote', 'resource_quote_comment', 'resource_quote_text', 'resource_year', 'tag', 'statistics', 
			'users', 'user_bibliography', 'user_groups', 'user_register', 'user_tags', 'resource_user_tags', 'subcategory');
//		if($tableContinue) // continuing after pause
//			$tables = array_slice($tables, array_search($tableContinue, $tables) + 1);
		foreach($tables as $value)
		{
			$change = $varchar = array();
			$basicTable = $value;
			$table = '`' . $this->config->WIKINDX_DB_TABLEPREFIX . "$basicTable`";
			$recordset = $this->db->query("SHOW COLUMNS FROM $table");
			while($row = $this->db->fetchRow($recordset))
			{
				if(($row['Type'] == 'varbinary(255)'))
					$varchar[] = '`' . $row['Field'] . '` `'  . $row['Field'] . '` varchar(255)';
				else if($row['Type'] == 'blob')
					$varchar[] = '`' . $row['Field'] . '` `'  . $row['Field'] . '` text';
			}
			$this->db->freeSet($recordset);
			if(!empty($varchar))
			{
				foreach($varchar as $new)
					$change[] = ' CHANGE ' . $new;
			}
			$this->db->query('ALTER TABLE ' . $table . join(',', $change));
			$tableArray = $basicTable . 'Array';
			if(isset($this->{$tableArray}))
			{
				foreach($this->{$tableArray} as $id => $array)
				{
					$updateArray = array();
					if($basicTable != 'config')
						$this->db->formatConditions(array(str_replace('_', '', $basicTable) . 'Id' => $id));
					foreach($array as $key => $value)
					{
						$updateArray[$key] = $value;
//						if(function_exists('iconv'))
//							$updateArray[$key] =  iconv("windows-1252", "utf-8", $value);
//						else
//							$updateArray[$key] = 
//							preg_replace("/([\x80-\xFF])/e", "chr(0xC0|ord('\\1')>>6).chr(0x80|ord('\\1')&0x3F)", $value);
					}
					if(!empty($updateArray))
						$this->db->update($basicTable, $updateArray);
				}
			}
		}
	}
/**
* Only the superadmin may update the database -- ask for login
*/
	private function confirmUpdateDisplay()
	{
		$this->session->setVar("setup_Template", 'default');
		$html = FACTORY_HTML::getInstance();
		$form = FACTORY_FORM::getInstance();
		$table = FACTORY_TABLE::getInstance();
		$pString = $this->html->p($this->html->b($this->messages->text("heading", "upgradeDB")));
		if(!$this->session->getVar('setup_Superadmin'))
		{
			$pString .= $html->p($this->messages->text("config", "upgradeDB1"));
			if($this->email)
				$pString .= $html->p($this->messages->text("config", "upgradeDB2", "&nbsp;({$this->email})"));
			else
				$pString .= $html->p($this->messages->text("config", "upgradeDB2"));
			$pString .= $html->p($this->messages->text("authorize", "logonSuperadmin"));
			$pString .= $form->formHeader("upgradeDBLogon");
			$pString .= $table->tableStart(FALSE, 0, 0, 0, "left", "50%");
			$pString .= $table->trStart();
			$pString .= $table->td($this->messages->text("user", "username") . ":&nbsp;&nbsp;");
			$pString .= $table->td($form->textInput(FALSE, "username"));
			$pString .= $table->trEnd();
			$pString .= $table->trStart(FALSE);
			$pString .= $table->td($this->messages->text("user", "password") . ":&nbsp;&nbsp;");
			$pString .= $table->td($form->passwordInput(FALSE, "password"));
			$pString .= $table->trEnd();
			$pString .= $table->trStart();
			$pString .= $table->td("&nbsp;");
			$pString .= $table->td($form->formSubmit(), FALSE, "right");
			$pString .= $table->trEnd();
			$pString .= $table->tableEnd();
			$pString .= $form->formEnd();
			$this->close($pString);
		}
		else
		{
			$pString .= $this->html->p('CURRENT MAX EXECUTION TIME: ' . ini_get("max_execution_time") . ' secs' . 
				$this->html->br() . 
				'CURRENT PHP MEMORY LIMIT: ' . ini_get("memory_limit"));
			$pString .= $html->p($this->messages->text("config", "upgradeDB1"));
			$pString .= $html->p($this->messages->text("config", "upgradeDB3"));
			$pString .= $form->formHeader("upgradeDB");
			$pString .= $html->p($form->formSubmit("upgradeDB"), FALSE, 'right');
			$pString .= $form->formEnd();
			$this->close($pString);
		}
	}
/**
* Intercept for initial configuration of database by admin and, if necessary, display admin configuration interface.
*/
	private function checkConfigTable()
	{
		if(!$this->rowExists())
		{
			include_once("core/modules/admin/CONFIGURE.php");
			$config = new CONFIGURE(TRUE);
			$config->insert = TRUE; // force table inserts in CONFIGURE
			if(isset($this->vars['action']) && $this->vars['action'] == 'admin_CONFIGURE_CORE')
				GLOBALS::buildOutputString($config->write());
			else
			{
// write preliminary stringLimit, write and superadmin to session and display configuration screen
				$this->session->setVar("setup_StringLimit", 40);
				$this->session->setVar("setup_Write", TRUE);
				$this->session->setVar("setup_Superadmin", TRUE);
// superadmin userId is always 1
				$this->session->setVar("setup_UserId", 1);
				$html = FACTORY_HTML::getInstance();
				GLOBALS::buildOutputString($config->init($html->p($this->messages->text("config", 
					"start"), "error", "center")));
				FACTORY_CLOSENOMENU::getInstance();
			}
		}
	}
/**
* At first initialization of wikindx database, populate the summary table and category table
*/
	private function populateTables()
	{
		$fields = array('databasesummaryTotalResources', 'databasesummaryTotalQuotes', 
			'databasesummaryTotalParaphrases', 'databasesummaryTotalMusings', 'databasesummaryDbVersion');
		$values = array('0', '0', '0', '0', WIKINDX_DB_VERSION);
		$this->db->insert('database_summary', $fields, $values);
		$this->db->insert('category', 'categoryCategory', 'General');
	}
/**
* Check if the config table has been initialized.
*
* Row must be returned as evidence of prior configuration.
*/
	private function rowExists()
	{
		$recordset = $this->db->select('config', "configTemplate");
		if(!$this->row = $this->db->fetchRow($recordset))
		{
			$this->db->freeSet($recordset);
			return FALSE;
		}
		$this->db->freeSet($recordset);
		return TRUE;
	}
/**
* Ensure all tables are set to UTF8 collation
*/
	private function setToUtf8()
	{
// Set database defaults to UTF8
		$this->db->query("ALTER DATABASE `" . $this->config->WIKINDX_DB . "` DEFAULT CHARACTER SET utf8 COLLATE utf8_unicode_ci");
		$tables = array('resource_attachments', 'bibtex_string', 'cache', 'category', 'collection', 'config', 'creator', 
			'custom', 'database_summary', 'import_raw', 'keyword', 'news', 'papers', 'publisher', 'resource', 
			'resource_category', 'resource_creator', 'resource_custom', 'resource_keyword', 
			'resource_misc', 'resource_musing', 'resource_musing_text', 'resource_text', 'resource_page', 
			'resource_paraphrase', 'resource_paraphrase_comment', 'resource_paraphrase_text', 
			'resource_quote', 'resource_quote_comment', 'resource_quote_text', 'resource_summary', 'resource_timestamp', 
			'resource_year', 'tag', 'users', 'user_bibliography', 'user_groups', 'user_register', 
			'user_tags', 'resource_user_tags', 'subcategory', 'statistics');
		foreach($tables as $basicTable)
		{
			$table = '`' . $this->config->WIKINDX_DB_TABLEPREFIX . "$basicTable`";
			$change[] = 'CONVERT TO CHARACTER SET utf8 COLLATE utf8_unicode_ci';
			$this->db->query("ALTER TABLE $table CONVERT TO CHARACTER SET utf8 COLLATE utf8_unicode_ci");
		}
	}
/**
* Special CLOSE function for pre v4.0 databases
* 
* @param string $pString
*/
	private function close($pString)
	{
		$string = '<html><head><title>WIKINDX Upgrade</title></head><body>' . $pString . '</body></html>';
		print $string;
		ob_end_flush();
		die;
	}
}
?>
